/*

In this version the possibility of off-centering the beam in t=0 has also been introduced. 
There are two more parameters to be input, xkick and ykick, which define the kick amplitude.

The tune spread introduced by space charge has also been updated: the tune shift of each
particle is calculated at each turn taking into account the local and instantaneous sigma
in the tune shift Laslett formula. Actually, according to the switch i_rot, it's possible to
get the space charge roatation at each turn either around the local centroids and by using
the local sigmas (i_rot=0), or around the full bunch centroid and by using the total sigmas
(i_rot=1). i_rot is the second last entry of the new input file.

The option of the electrons moving in a sinusoidal field is here included - it
corresponds to i_dip=2 and takes the input value for the solenoid field (bsol, last line
in the input file)

The option of the proton space charge has been added - by using the routine ffrank, we make
each bunch slice act on each proton in the slice, too. A new switch has been introduced,
i_space, which is a value to be input in the input file. The most updated input file
is to be found in the directory ~/Elcloud/HEADTAIL/DATA/PIC/TUNESPR and it's nobb0quacin.cfg
- sisexp.cfg is in the ../PIC/EXPER directory 
(input values: 37)

   !!!! This file has two executables: !!!!

   xcloud: electron-cloud problems oriented - it has NPR=300000 NEL=100000 NBIN=50
   ximped:  impedance problems oriented - it has NPR=1000000 NEL=1 NBIN=70

Routine new_fraction is introduced, in which 500 macroelectrons are newly initialized at
each interaction slice-cloud. This is done only when a solenoid field is acting. It models
the fact that electrons in the solenoid field are actually moving longitudinally. We
take these numbers: electron cooler section 3m long, bunch 5m long, electron velocity
is 0.145c and proton (or D+) velocity 0.947c. We easily find Deltat_b = 5/0.947c = 17.6ns 
-> during one full bunch pass the electrons will have advanced by 
Deltas_e = 0.145c Deltat_b = 0.765m. Therefore 1/4 of the total cooler length. This means 
that at the end of the interaction, 1/4 of the cloud keeps actually a pregressively 
"shorter memory" of the previous interaction. If after 50 slices 1/4 of the macroelectrons 
(25000) will have this partial memory, the effect can be achieved if at each slice we 
regenerate 25000/50 = 500 electrons.

Last improvement has been made with Daniel Schulte, he has basically made the PIC field 
calculation faster by changing the modules fourtrans with improved versions (fourtrans2 and
fourtrans3) which link to the file fourtrans2.c, included in the present program, and to 
routines from the fftw library which has also been copied in the fftw directory from my
home directory (7/12/2001).

As the amplitude detuning has been also introduced, there is a new switch i_oct which sets it
working or not. It is the last entry in the input file (line 38). If amplitude detuning is
turned on, its parametrization is given in the 'init_values' routine of this code and actually
reflects the parametrization expected for SPS (from Rogelio's note).

11/04/2002
We have introduced a new option: i_dip=3. The electrons move in the strong combined functions
magnetic field, like in PS. With the assumption of strong field, we have simply confined the
electron motion on trajectories that follow the field lines.

29/04/2002
The new flag i_incoh has been introduced to study the instability free of the monopole mode
as suggested by Evgueni Perevedentsev. The bunch phase space centroid is subtracted turn by
turn and slice by slice from the particle phase space coordinates to concentrate only on the
e-cloud induced emittance growth

The flag elflag defines the electron distribution:
1- Rectangular 2- Elliptical 3- Stripe-like rectangular 4- Parabolic

Number of entries of the input file: 40 !!! new entries to be added:

Coherent_centroid_motion_(0->off_1->on):
Electron_distribution(1->Rect_2->Ellipt_3->Stripe_4->Parab):

for most updated input file look into:

/afs/cern.ch/user/g/grumolo/Elcloud/HEADTAIL/DATA/PIC/PEREVED/eugeni*.cfg
/afs/cern.ch/user/g/grumolo/Elcloud/HEADTAIL/DATA/PIC/PS/COMBIN/cfcurr8ott.cfg

3/06/2002
new flag i_coupl to have possibly linear xy-coupling in the machine. The parameter key_xy is
also defined to tune the strength of the coupling. To be input on last line of the input file.

Number of entries of the input file: 42 !!! new entries to be added:

Linear_coupling_switch(1->on_0->off):                            1
Linear_coupling_coefficient_[1/m]:                               0.015

17/06/2002
->new<- to take into account horizontal dispersion: the value is input in the variable dispx 
(43rd line in the input file !!), and the contribution of dispersion comes in when the
interaction between bunch and cloud is calculated. This introduces a x-z correlation first, and
obviously it changes the oscillation frequency of the electrons because it changes the 
horizontal rms-size value.
Also the 2-stripes distribution is included for electrons, which takes into account how electrons
are actually distributed inside a dipole field.
Number of entries of the input file: 45 !!! new entries:

Average_dispersion_function_in_the_ring_[m]:                     2.0
Position_of_the_stripes_[units_of_sigmax]:                       3.0
Width_of_the_stripes_[units_of_sigmax]:                          0.5

and the line on the electron distribution becomes:

Electron_distrib_(1->Rect_2->Ellip_3->[1_strp]_4->[2_strp]_5->Parab):

08/08/2002
A kick in the longitudinal direction has been introduced, after having changed the bining in
such a way that the code can deal with bunch dipole oscillations in z and variations of sigma_z 
during the evolution (to model for example a debunching beam). New line in the input file is 
therefore needed:

Kick_in_the_longitudinal_direction_[m]:                          0.03

If isyn==2 we also give the possibility to the bunch to de-bunch. Synchrotron motion is off and
the particles follow their longitudinal straight trajectories according to their initial momenta.

13/08/02

Changing the transverse impedance: the transverse force due to the broad band impedance has been
adapted to a case of flat chamber (the two limit cases are now possible: round chamber i_pipe=0
(a=b) or very flat chamber i_pipe=1 (a>>b))
this introduces one more line in the input file to set i_pipe, but also two lines of the input
need to be taken out because we have one single set of parameters for the trasverse impedance,
which will be used both for horizontal and vertical kicks + the flat chamber correction.

OLD:

Switch_for_wake_fields:                                          0
Res_frequency_of_broad_band_resonator_[GHz]:                     1.3
Horizontal_quality_factor:                                       1.
Vertical_quality_factor:                                         1.
Shunt_impedance_x_[MOhm/m]:                                      5.
Shunt_impedance_y_[MOhm/m]:                                      10.

NEW:

Switch_for_wake_fields:                                          0
Switch_for_pipe_geometry_(0->round_1->flat):                     1
Res_frequency_of_broad_band_resonator_[GHz]:                     1.3
Transverse_quality_factor:                                       1.
Transverse_shunt_impedance_[MOhm/m]:                             10.

15/08/02

It is possible to store the bunch shape every n_diag turns. This makes sense when we are in 
debunching mode or in bunched mode with a bunch not well matched to the bucket. One more input
parameter:

Number_of_turns_between_two_bunch_shape_acquisitions:            20

Longitudinal wake field has been introduced due to a resonator. We want to model the parasitic
effect of a cavity on a single bunch. We can neglect the turn-to-turn effect, since the quality
factor is such (~150) that the field has completely decayed when the bunch goes again through
the same cavity where it was generated (Q=5000 is at the limit if this approximation for a 
frequency around 200 MHz). Again we need to input three numbers to model this resonator:

Res_frequency_of_broad_band_resonator_[GHz]:                     1.3
Transverse_quality_factor:                                       1.
Transverse_shunt_impedance_[MOhm/m]:                             10.
Res_frequency_of_longitudinal_resonator_[MHz]:                   200
Longitudinal_quality_factor:                                     140.
Longitudinal_shunt_impedance_[MOhm]:                             5.

20/08/02

Fields with conducting boundary conditions on a rectangular box can be calculated instead of
the simple periodic condition in x and y. The method consists in unfolding the charge distribution
on a double-sized grid on which the image charges are placed. The routine image_unfold has been
introduced to accomplish this. 
One more input needed:

08/05/04

FZ, introduce two new isyn options: 
isyn=6 for TESLA DR where momentum compaction & rf are localized at one point of the ring
isyn=7 for CLIC, NLC, LHC where rf is localized at one point of the ring 


Boundary_condition_(0->Periodic_1->Conducting):                  1


*/

/* Include all facilities   */

#include  <string.h>
#include  <stdio.h>
//#include  <iostream.h>  
#include  <math.h>
#include  <libio.h>
#include  <time.h>
#include  <stdlib.h>
#include  <complex.h>  /* with this package one can define complex variables,
                          if needed; for instance, complex<double> z         */

/* Definition of constants  */
/* Physical constants....   */

#define  PI      3.14159265358979
#define  C       2.99792458e8       /* velocity of light          [m/s]      */
#define  E       1.6021892e-19      /* electron charge            [Cb]       */
#define  MU      1.66055e-27        /* proton mass                [kg]       */
#define  ME      9.109534e-31       /* proton-positron mass       [kg]       */
#define  EPS0    8.8542e-12         /* dielectric constant        [C^2/Jm]   */
#define  RP      1.5458775e-18      /* classical proton radius    [m]        */
#define  RE      2.817938e-15       /* classical electron radius  [m]        */
#define  I0      31.356e6           /* ... for the tune shift..   [A]        */
#define  ELMS    9.109534e-31       /* electron mass              [Kg]       */

/* Parameters for the computation... */

//
// #define  NPR     1000000            /* Number of macroparticles (bunch)
//                                      to be used                         */
// #define  NPR1    90000              /* Maximum number of macroparticles
//                                       in a slice                         */
// #define  NEL     1                  /* Number of macroelectrons (cloud)
//                                       to be used                         */
// #define  NBIN    200                /* Number of 'slices' in which each
//                                      single bunch is sub-divided        */
// #define  NDC     128                /* Number of cells to produce a
//				       distribution function              */

//
// #define  NPR     300000            /* Number of macroparticles (bunch)
//                                      to be used                         */
// #define  NPR1    90000              /* Maximum number of macroparticles
//                                       in a slice                         */
// #define  NEL     100000                  /* Number of macroelectrons (cloud)
//                                       to be used                         */
// #define  NBIN    70                /* Number of 'slices' in which each
//                                      single bunch is sub-divided        */
// #define  NDC     128                /* Number of cells to produce a
//				       distribution function              */
				

#define  NPR     300000             /* Number of macroparticles (bunch)
                                       to be used                         */
#define  NPR1    90000              /* Maximum number of macroparticles
                                       in a slice                         */
#define  NEL     100000          /* Number of macroelectrons (cloud)
                                       to be used                         */
#define  NBIN    70                 /* Number of 'slices' in which each
                                      single bunch is sub-divided        */
#define  NDC     128                /* Number of cells to produce a
                                       distribution function              */
                                      

#define  NR_END    1
#define  FREE_ARG  char*


/* type definition  */

typedef struct /* GRID */
{
  int n_cell_x,n_cell_y;
  int timestep;
  double delta_x,delta_y;
  double offset_x,offset_y;
  double min_x,max_x,min_y,max_y;
  double cut_x,cut_y,cut_z;
  double step,n_macro_1,n_macro_2,step1,step2,scal_step[2];
  float rho_x_1,rho_y_1,rho_sum_1,rho_x_2,rho_y_2,rho_sum_2;
  double rho_factor;
  int n_m_1,n_m_2;
  double *rho1,*rho2,*phi1,*phi2,*dist,*temp;
  int integration_method,distribution,interpolation;
} GRID;

/* Definition of the external variables   */

FILE  *bunch_pr             ,
      *headtail_pr          ,
      *prot_phase           ,
      *prot_inphase         ,
  *prot_ini            ,
      *samp_prot            ,
      *prot_distr           ,
      *track_pr ;

char  filename[80]          ;

long  iind[NBIN]            ,   /* iind[i] gives the number of macroparticles in
                                   the i-th slice of bunch                          */
      indexp[NBIN][NPR1]    ,   /* in the bunch slicing, indexp[i][j] contains the
                                   original bunch index of the particle that
                                   has index j in the i-th slice                    */ 
      hmain                 ,   /* down to mmain -> indices used in main for loops  */
      imain                 ,
      jmain                 ,
      kmain                 ,
      lmain                 ,
      mmain                 ,
  contax ,    
  mark[NPR] ,
      elflag                ,   /* flag for the e-distribution: see above           */
      it                    ,   /* index used for the main cycle over the total 
                                   number of steps: turns * kicks                   */
      ind                   ,
      nturn                 ,   /* number of simulated turns -> input parameter     */
      nkick                 ,   /* number of kicks per turn  -> input parameter     */
      nstep                 ,   /* total number of steps: nturn * nkick             */
      isyn                  ,   /* flag for the synchrotron motion: input parameter */
      ipr_pos               ,   /* flag for proton/positron bunch: input parameter  */
      imesize               ,   /* scale factor for macroelectron size: input 
                                   parameter                                        */
      ielsizex              ,   /* multiplication factor for the e- cloud x-size: 
                                   input parameter                                  */
      ielsizey              ,   /* multiplication factor for the e- cloud y-size: 
                                   input parameter                                  */
      nsigmaz               ,   /* number of sigma_z to be taken into account to 
				   represent the Gaussian bunch (+/- nsigmaz*szt)   */
      ili                   ,   /* flag for the function ffrank: it has to be =! 1  */
      iwake_flag            ,   /* flag for the wake field: input parameter         */
      ishift                ,   /* flag to activate the tune shift option           */
      sl_ind                ,   /* it is used as slice index when the incoherent 
                                   space charge tune shift is added                 */
      i_method              ,   /* flag to choose the field calculation method      */
      i_rot                 ,   /* switch for the space charge tune rotation        */
      i_space               ,   /* Flag for the space charge forces                 */
      i_oct                 ,   /* Flag for the action dependent tunes              */
      i_dip                 ,   /* flag to switch off the electron horizontal motion
                                   in order to simulate their motion in a strong
                                   vertical dipole field                            */
      i_incoh               ,   /* flag to investigate on the monopole mode as
                                   suggested by Evgueni Perevedentsev               */
      i_coupl               ,   /* flag to switch on and off linear coupling        */
      i_sext                ,   /* flag to switch on and off sextupolar kick        */
      i_loss                ,   /* switch to activate beam losses                   */
      i_pipe                ,   /* flag for pipe shape in effective wake calculation
                                   0 -> circular (a=b)  1 -> flat  (a>>b)    
				   3 -> resistive_wall  5 ->collim_geometric_wake  
				   6 ->collim_resistive_wake           		*/
      i_bound               ,   /* switch for boundary conditions
                                   0 -> open space
                                   1 -> rectangular box                             */
      i_ecool               ,   /* switch for an electron cooler tuned on the beam  
                                   0 -> electron cloud or mis-tuned electron cooler
                                   1 -> electron cooler                             */
      i_phas_adv            ,   /* switch for random phase advance                  */
      i_damp                ,   /* switch for the damper                            */
      n_cool                ,   /* number of sections in which the e-cooler gets
                                   subdivided                                       */
      n_diag                ,   /* every how many turns the bunch shape is stored   */
      hh                    ,   /* Harmonic number of sinusoidal bucket             */
      npr_eff               ;   /* number of bunch particles lost turn by turn      */

double ycoff;
double xpr[NPR]             ,   /* horizontal coordinate for protons or positrons   */
       ypr[NPR]             ,   /* vertical coordinate for protons or positrons     */
       xpp[NPR]             ,   /* dx/ds  for protons or positrons                  */
       ypp[NPR]             ,   /* dy/ds  for protons or positrons                  */
       sz[NPR]              ,   /* longitudinal coordinate for protons or positrons */
       dp[NPR]              ,   /* relative momentum spread for protons or 
                                   positrons                                        */
       xe[NEL]              ,   /* horizontal coordinate for electrons              */
       ye[NEL]              ,   /* vertical coordinate for electrons                */
       xpe[NEL]             ,   /* dxe/dt  for electrons                            */
       ype[NEL]             ,   /* dye/dt  for electrons                            */
  //xe0[NEL]             ,
  //ye0[NEL]             ,
       zs[NBIN]             ,   /* longitudinal position of each bunch slice        */
       npr[NBIN]            ,   /* number of particles in each slice                */
       sxs[NBIN]            ,   /* horizontal rms value of each slice               */
       sys[NBIN]            ,   /* vertical rms value of each slice                 */
       xs[NBIN]             ,   /* horizontal mean offset of each slice             */
       ys[NBIN]             ,   /* vertical mean offset of each slice               */
       xps[NBIN]            ,   /* horizontal mean dx/ds of each slice              */
       yps[NBIN]            ,   /* vertical mean dy/ds of each slice                */
       indx[NDC]            ,   /* grid points for the horizontal distribution      
                                   of electrons                                     */
  //hde[NDC]             ,   /* horizontal distribution of electrons after having been kicked   */
  //indy[NDC]            ,   /* grid points for the vertical distribution of electrons          */
  //vde[NDC]             ,   /* vertical distribution of electrons after having been kicked     */

       indxp[NDC]           ,   /* grid points for the bunch long. distribution     */
       hdp[NDC]             ,   /* Bunch longitudinal distribution                  */

  //indyp[NDC]           ,   /* grid points for the vertical distribution of the bunch particles  */
  //vdp[NDC]             ,   /* vertical distribution of bunch particles before interacting with the particles */
  //indxpp[NDC]          ,   /* grid points for the distribution of horizontal momenta of the bunch particles  */
  //hpdp[NDC]            ,   /* distribution of horizontal momenta of the bunch particles before interacting with the electrons  */
  //indypp[NDC]          ,   /* grid points for the distribution of vertical momenta of the bunch particles    */
  //vpdp[NDC]            ,   /* distribution of vertical momenta of the bunch particles before interacting with the electrons    */

       vprx[NPR1]           ,   /* temporary vector that contains all the x's of 
                                   the bunch particles in a slice                   */
       vpry[NPR1]           ,   /* temporary vector that contains all the y's of 
                                   the bunch particles in a slice                   */
       vprxp[NPR1]          ,   /* temporary vector that contains all the xp's of 
                                   the bunch particles in a slice                   */
       vpryp[NPR1]          ,   /* temporary vector that contains all the yp's of 
                                   the bunch particles in a slice                   */

  /* now the declaration of the input parameters starts....   
     it goes up to nus = synchrotron tune                                           */

       sz0                  ,   /* [m] initial rms-bunch length                     */
       dp0                  ,   /* initial rms-momentum spread                      */
       sx0                  ,   /* [m] initial rms hor. beam size                   */
       sx0_cd               ,   /* [m] rms hor. beam size dispersion corrected      */
       sy0                  ,   /* [m] initial rms vert. beam size                  */
       npr0                 ,   /* number of particles in the bunch                 */
       betax0               ,   /* [m] average hor. beta function around the ring   */
       betay0               ,   /* [m] average vert. beta function around the ring  */
       dispx                ,   /* [m] average dispersion function in the ring      */
       dispx2               ,   /* [m] average dispersion function at sextupoles    */
       dcloud               ,   /* average number of electrons per m^3 in the ring  */
       circ                 ,   /* [m] ring circumference                           */
       gammar               ,   /* relativistic gamma of the bunch particles        */
       htune                ,   /* horizontal tune of the machine                   */
       vtune                ,   /* vertical tune of the machine                     */
       csix                 ,   /* horizontal chromaticity of the machine           */
       csiy                 ,   /* vertical chromaticity of the machine             */
       theta_coll           ,    
       b_coll               ,
       h_coll               ,
       gainx                ,
       gainy                ,
       noisex               ,
       noisey               ,
       dampx                ,
       dampy                ,
       randomnumber        ,
       q_secx               ,   /* second order horizontal chromaticity             */
       q_secy               ,   /* second order vertical chromaticity               */
       alpha                ,   /* momentum compaction factor of the machine        */
       nus                  ,   /* synchrotron tune                                 */
       xkick                ,   /* [m] Amplitude of <the horizontal kick in t=0      */
       ykick                ,   /* [m] Amplitude of the vertical kick in t=0        */
       bsol                 ,   /* [T] Strength of the solenoid field               */
       bdip                 ,   /* [T] Strength of the dipole field in a combined 
                                   functions magnet (like in the PS)                */
       bgrad                ,   /* [T/m] Gradient of the magnetic field in a 
                                   combined functions magnet (like in the PS)       */
       rpipex               ,   /* horizontal semiaxis of the beam pipe: gives a 
                                   x-size to the e- cloud                           */
       rpipey               ,   /* vertical semiaxis of the beam pipe: gives a
                                   y-size to the e- cloud                           */
       gridextx             ,   /* Horizontal extension of the grid for the fields
                                   PIC calculation                                  */
       gridexty             ,   /* Vertical extension of the grid for the fields
                                   PIC calculation                                  */
       strpos               ,   /* position of the stripes in units of sigma_x      */
       strsig               ,   /* width of the stripes in units of sigma_x         */
       eta                  ,   /* slip factor: alpha - 1/gammar^2                  */
       omega0               ,   /* bunch revolution frequency                       */
       omegas               ,   /* synchrotron frequency                            */
  mass                 ,   /* these are used if the particles are ions and they*/
  charge               ,   /*are defined in the subroutine 'init_values'      */
       factor1              ,   /* auxiliary parameter for the force calculation    */
       factor2              ,   /* auxiliary parameter for the force calculation    */
       fscale1              ,   /* auxiliary parameter for the force calculation    */
       fscale2              ,   /* auxiliary parameter for the force calculation    */
       eel                  ,   /* auxiliary parameter for the force calculation    */
       epr                  ,   /* auxiliary parameter for the force calculation    */
       wakefac              ,   /* factor for the wake field kick                   */
       tstep                ,   /* time step during the interaction bunch-cloud     */
       dlstep               ,   /* s-distance between two kick points               */
       tstepp               ,   /* time step between two kick points                */
       qp0                  ,   /* number of bunch particles in one macroparticle   */
       qe0                  ,   /* number of electrons in one macroelectron         */
       p0                   ,   /* average momentum of the bunch particles          */
       sx2                  ,   /* finite macroelectron horizontal size             */
       sy2                  ,   /* finite macroelectron vertical size               */

       key_xy               ,   /* [1/m] strength of linear coupling                */
       k_2s                 ,   /* [1/m^2] strength of sextupole                    */
       cxa                  ,   /* cxa -> ssa: transport parameters for the phase   */
       sxa                  ,   /* space coordinates of the bunch particles (linear */
       cya                  ,   /* transport + chromaticity kick in the transverse  */
       sya                  ,   /* plane, linear synchrotron motion longitudinally) */

       gammaadv0            ,
       gammaadv             ,    /* #####################*/
       hadv                 , 
       vadv                 , 
       phadv[200]           ,

       csa                  ,
       ssa                  ,
       dtunxmax             ,   /* maximum space charge horizontal tune shift in 
                                   the bunch                                        */
       dtunymax             ,   /* maximum space charge vertical tune shift in
                                   the bunch                                        */
       dtunx                ,
       dtuny                ,
       lambda               ,   /* local variable to store the exponential factor
                                   for the tune shift modulation along the bunch    */

       cent_x               ,   /* cent_*: local variables to store the centroid    */
       cent_y               ,   /* position and slope of each slice used for the    */ 
       cent_xp              ,   /* incoherent space charge tune spread              */
       cent_yp              ,
       xx1                  ,   /* xx1->yyp1: local variables used in the particles */
       xxp1                 ,   /* transport around the ring                        */
       yy1                  ,
       yyp1                 ,
       epsx                 ,   /* single particle horizontal emittance variable    */
       epsy                 ,   /* single particle vertical emittance variable      */
       qpxx                 ,   /* Q_x = Q_x0 + eps_x * qpxx + eps_y * qpxy         */
       qpyy                 ,   /* Q_y = Q_y0 + eps_x * qpxy + eps_y * qpyy         */
       qpxy                 ,
       bin_frac             ,   /* fraction of bunch particles that are binned      */

       xave                 ,   /* average horizontal offset of the beam            */
       yave                 ,   /* average vertical offset of the beam              */
       zave                 ,   /* average longitudinal offset of the beam          */
       dpmean               ,   /* average long. momentum offset of the beam        */
       xpave                ,   /* average beam offset in dx/ds                     */
       ypave                ,   /* average beam offset in dy/ds                     */
       sxsize               ,   /* square of horizontal rms bunch size              */
       sysize               ,   /* square of vertical rms bunch size                */
       szsize               ,   /* longitudinal rms bunch size                      */
       sxpsize              ,
       sypsize              ,
    sxs_ave            ,      /* average beam size, w/o coherent motion        */
    sys_ave            ,
   
       timep                ,   /* time at which the kick occurs                    */
       xemit                ,   /* horizontal emittance                             */
       yemit                ,   /* vertical emittance                               */
       invarx               ,   /* Courant-Snyder horizontal invariant              */
       invary               ,   /* Courant-Snyder vertical invariant                */
       corryz               ,   /* y-z correlation                                  */
       npr1                 ,   /* number of protons contained in one slice         */
       xoff                 ,   /* slice horizontal off-center                      */
       yoff                 ,   /* slice vertical off-center                        */
       sx                   ,   /* slice horizontal rms size                        */
       sy                   ,   /* slice vertical rms-size                          */
       xta                  ,
       yta                  ,
       zstep                ,   /* length of a bunch slice                          */
       zi                   ,   /* longitudinal negative coordinate (head of the
                                   bunch is zi = 0) for wake field                  */
       x1                   ,   /* macroelectron x position at which the force 
                                   from each slice is calculated                    */ 
       ips1                 ,   /* macroelectron y position at which the force 
                                   from each slice is calculated                    */
       pre                  ,   /* coefficient for the y-component of the force     */ 
       pim                  ,   /* coefficient for the x-component of the force     */
       kick_xe              ,   /* velocity x-kick from each slice on each 
                                   macroelectron                                    */ 
       kick_ye              ,   /* velocity y-kick from each slice on each 
                                   macroelectron                                    */
       xof2                 ,   /* macroelectron x-position                         */
       yof2                 ,   /* macroelectron y-position                         */
       x2                   ,   /* x-position of the bunch particle on which the 
                                   macroelectron acts                               */
       y2                   ,   /* y-position of the bunch particle on which the 
                                   macroelectron acts                               */
       xof3                 ,   /* initial x-position of the macroelectron - used 
                                   to clean out fluctuations                        */ 
       yof3                 ,   /* initial y-position of the macroelectron - used
                                   to clean out fluctuations                        */
       xta2                 ,
       yta2                 ,
       pre2                 ,   /* coefficient for the x-component of the force     */
       pim2                 ,   /* coefficient for the y-component of the force     */
       kick_x               ,   /* x-kick on each bunch particle                    */
       kick_y               ,   /* y-kick on each bunch particle                    */
       kick_z               ,   /* z-kick from on each bunch particle               */
       z0_kick              ,   /* z-kick from each macroelectron on each 
                                   bunch particle                                   */
       kick_xp_sc           ,   /* dx/ds-kick from each bunch slice on each 
                                   bunch particle                                   */
       kick_yp_sc           ,   /* dy/ds-kick from each bunch slice on each 
                                   bunch particle                                   */
       x0                   ,   /* x-coordinate of a bunch particle after having 
                                   been kicked, before transport to the next kick   */
       xp0                  ,   /* dx/ds of a bunch particle after having been 
                                   kicked and before transport to the next kick     */
       ips0                 ,   /* y-coordinate of a bunch particle after having 
                                   been kicked, before transport to the next kick   */
       yp0                  ,   /* dy/ds of a bunch particle after having been 
                                   kicked and before transport                      */
       szz0                 ,   /* longitudinal coordinate of a bunch particle 
                                   after having been kicked, before transport to
                                   the next kick                                    */
       dpp0                 ,   /* momentum spread of a bunch particle after having
                                   been kicked, before transport to the next kick   */
       r11                  ,   /* r11 -> r44, coefficients of the transport 
                                   matrix for the transverse phase space coordinates
                                   between two kicks                                */
       r12                  ,
       r21                  ,
       r22                  ,
       r33                  ,
       r34                  ,
       r43                  ,
       r44                  ,
       merit                ,    /* quality factor of the broad band impedance      */
       freqr                ,    /* resonant frequency of the broad band resonator
                                    to be expressed in GHz (x and y assumed equal)  */
       zetat                ,    /* shunt impedance of the braod band resonator
                                    to be expressed in MOhm/m                       */
       meritz               ,    /* quality factor of the longitudinal impedance    */
       freqrz               ,    /* resonant frequency of the longitudinal resonator
                                    to be expressed in MHz                          */
       rshz                 ,    /* shunt impedance of the braod band resonator
                                    to be expressed in MOhm                         */
       omegar               ,    /* auxiliary parameters for transverse wake        */
       omegabar             , 
       alpha                ,
       omegarz              ,    /* auxiliary parameters for longitudinal wake      */
       omegabarz            , 
       alphaz               ,
       voltm                ,    /* Maximum voltage in sinusoidal bucket            */  
       wrf                  ,    /* Radio-frequency of sinusoidal bucket            */
       coeff3               ,    /* Auxiliary parameters for longitudinal motion    */
       coeff4               ,    /* in sinusoidal bucket  */
       coeff5               ,
       coeffspch            ,    /* coefficient for longitudinal space charge in a
				    sinusoidal bucket                               */
       k1                   ,
       k2                   ,
       k3                   ,
       k4                   ,
       kp1                  ,
       kp2                  ,
       kp3                  ,
       kp4                  ,
       dl_cool              ;    /* electron cooler length                          */
           
/* For sake of clarity we specify here that the broad band model that we've assumed 
   considers an impedance (omegar = 2*PI*freqr*10^(-9)): 

      Z = omegar/omega * zetat/[1+i merit (omegar/omega - omega/omegar)].
   
   If the impedance is: 

      Z = c/omega * R_s/[1+i merit (omegar/omega - omega/omegar)], 
   
   then one has to convert R_s into our input parameter by means of: 

   zetat=c/omegar*R_s  
   
   zetat [Ohm/m] and R_s [Ohm/m^2]                                                    

   In the longitudinal plane:

      Z|| = rshz/[1+i meritz (omegarz/omega - omega/omegarz)] 
                                                                                     */
/*  Global declarations... 
    function that evaluates the electric mutual force between bunch particles
    and electrons has to be included                                                 */

extern void ffrank (double*, double*, double*, double*, double*, double*, 
                    double*, double*, long*);

/******************************************************************************
 *** Subroutine     : read_data                                             ***
 *** Effect         : Define main parameters by reading data from specified ***
 ***                  configuration-file                                    ***
 *** Parameters     : none                                                  ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used : none                                                  ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

read_data ()

{ char  cfg_filename[30], dummy_string[64];
  FILE  *cfg_file_ptr;

  //printf ("\n\n    Please specify name (without extension) of desired") ;
  //printf ("\n    configuration-file : ") ;

  //scanf ("%s",filename) ;
 
   //strcpy (filename,"ecsdampA");
   strcpy (filename,"ecjlast");
   //strcpy (filename,"ecjlastali");

  strcpy (cfg_filename,filename);
  strcat (cfg_filename,".cfg");

  cfg_file_ptr = fopen(cfg_filename,"r");
  if (cfg_file_ptr == NULL)
     { printf ("\n\n    Configuration-file does not exist.\n") ;
       end () ; }

  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&ipr_pos);
  // printf ("ipr_pos=%d\n",ipr_pos);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&dcloud);
  //printf ("dcloud=%lf\n",dcloud);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&npr0);
  // printf ("npr0=%lf\n",npr0);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&betax0);
  // printf ("betax0=%lf\n",betax0);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&betay0);
  // printf ("betay0=%lf\n",betay0);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&sz0);
  // printf ("sz0=%lf\n",sz0);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&sx0);
  // printf ("sx0=%lf\n",sx0);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&sy0);
  //  printf ("sy0=%lf\n",sy0);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&dp0);
  // printf ("dp0=%lf\n",dp0);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&nus);
  // printf ("nus=%lf\n",nus);
  fscanf (cfg_file_ptr,"%s",dummy_string);  
  fscanf (cfg_file_ptr,"%lf",&alpha);
  // printf ("alpha=%lf\n",alpha);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&circ);
  // printf ("circ=%lf\n",circ);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&gammar);
  //printf ("gammar=%lf\n",gammar);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&nkick);
  //  printf ("nkick=%d\n",nkick);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&nturn);
  //  printf ("nturn=%d\n",nturn);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&ielsizex);
  //  printf ("ielsizex=%d\n",ielsizex);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&ielsizey);
  // printf ("ielsizey=%d\n",ielsizey);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&nsigmaz);
  // printf ("+/-nsigmaz=%d\n",nsigmaz);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&htune);
  // printf ("htune=%lf\n",htune);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&vtune);
  // printf ("vtune=%lf\n",vtune);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&csix);
  // printf ("csix=%lf\n",csix);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&csiy);
  // printf ("csiy=%lf\n",csiy);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&isyn);
  // printf ("isyn=%d\n",isyn);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&imesize);
  // printf ("imesize=%d\n",imesize);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&iwake_flag);
  //printf ("iwake_flag=%d\n",iwake_flag);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_pipe);
  // printf ("i_pipe=%d\n",i_pipe);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&freqr);
  // printf ("freqr=%lf\n",freqr);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&merit);
  // printf ("merit=%lf\n",merit);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&zetat);
  // printf ("zetat=%lf\n",zetat);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&freqrz);
  // printf ("freqrz=%lf\n",freqrz);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&meritz);
  // printf ("meritz=%lf\n",meritz);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&rshz);
  // printf ("rshz=%lf\n",rshz);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&ishift);
  // printf ("ishift=%d\n",ishift);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_method);
  // printf ("i_method=%d\n",i_method);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_dip);
  // printf ("i_dip=%d\n",i_dip);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&xkick);
  // printf ("xkick=%lf\n",xkick);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&ykick);
  // printf ("ykick=%lf\n",ykick);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_space);
  // printf ("i_space=%d\n",i_space);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_rot);
  // printf ("i_rot=%d\n",i_rot);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&bsol);
  //printf ("bsol=%lf\n",bsol);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_oct);
  // printf ("i_oct=%d\n",i_oct);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_incoh);
  // printf ("i_incoh=%d\n",i_incoh);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&elflag);
  // printf ("elflag=%d\n",elflag);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_coupl);
  // printf ("i_coupl=%d\n",i_coupl);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&key_xy);
  // printf ("key_xy=%lf\n",key_xy);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&dispx);
  // printf ("dispx=%lf\n",dispx);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&strpos);
  // printf ("strpos=%lf\n",strpos);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&strsig);
  // printf ("strsig=%lf\n",strsig);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&z0_kick);
  // printf ("z0_kick=%lf\n",z0_kick);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&n_diag);
  // printf ("n_diag=%d\n",n_diag);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&voltm);
  //printf ("z0_kick=%lf\n",voltm);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&hh);
  //printf ("n_diag=%d\n",hh);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_sext);
  //  printf ("n_diag=%d\n",i_sext);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&k_2s);
  /// printf ("n_diag=%lf\n",k_2s);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&dispx2);
  //  printf ("n_diag=%lf\n",dispx2);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_loss);
  // printf ("i_loss=%d\n",i_loss);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&q_secx);
  // printf ("n_diag=%lf\n",q_secx);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&q_secy);
  /// printf ("n_diag=%lf\n",q_secy);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_bound);
  // printf ("i_bound=%d\n",i_bound);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_phas_adv);
  // printf ("i_phase_adv=%d\n",i_phas_adv);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_ecool);
  // printf ("i_ecool=%d\n",i_ecool);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&dl_cool);
  // printf ("dl_cool=%lf\n",dl_cool);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&theta_coll);
  // printf ("theta_coll=%lf\n",theta_coll);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&b_coll);
  // printf ("b_coll=%lf\n",b_coll);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&h_coll);
  // printf ("h_coll=%lf\n",h_coll);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%d",&i_damp);
   printf ("i_ecool=%d\n",i_damp);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&gainx);
  // printf ("gainx=%lf\n",gainx);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&noisex);
  // printf ("noisex=%lf\n",noisex);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&gainy);
  // printf ("gainy=%lf\n",gainy);
  fscanf (cfg_file_ptr,"%s",dummy_string);
  fscanf (cfg_file_ptr,"%lf",&noisey);
  // printf ("noisey=%lf\n",noisey);
 
  if(i_ecool==1)
    { i_dip = 2;
      printf("\n\n\t --> i_dip set to 2, E-cooler is tuned on the beam\n");
    }
  if(i_dip==2)
    { elflag = 2;
      if(i_ecool==0)
        printf("\n\n\t elflag set to 2, E-cooler and beam are mis-tuned\n"); 
      else
        printf("\n\n\t elflag set to 2, E-cooler is tuned on the beam\n");
    }

  printf ("\n\n\t\tConfiguration-file loaded successfully.\n") ;

}

/******************************************************************************
 *** Subroutine     : init_values                                           ***
 *** Effect         : Initialize all parameter values, necessary for        ***
 ***                  calculation.                                          ***
 *** Parameters     : none                                                  ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used :                                                       ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

init_values ()

{ double area;

    eta = alpha - 1/gammar/gammar;
    omega0 = C/circ*2*PI;
    omegas = nus*omega0;
    tstep = 2.*(double)nsigmaz*sz0/(double)NBIN/C;
    factor1 = RE*C*sqrt(2*PI)/tstep;
    fscale1 = factor1/sqrt(2*PI);
    dlstep = circ/(double)nkick;
    tstepp = dlstep/C;
    npr_eff = 0;
    omegar = 2*PI*freqr*1e9;
    alpha = omegar/(2*merit);
    omegabar = sqrt(omegar*omegar - alpha*alpha);
    omegarz = 2*PI*freqrz*1e6;
    alphaz = omegarz/(2*meritz);
    omegabarz = sqrt(omegarz*omegarz - alphaz*alphaz);
    wrf = (double)hh*omega0;


    charge=1.;


    switch(ipr_pos){
    case 1: 

      factor2 = RP*sqrt(2*PI)/gammar/tstep;
      p0 = MU*gammar*C;
      break;

    case 2:

      factor2 = RE*sqrt(2*PI)/gammar/tstep;
      p0 = ME*gammar*C;
      break;

    case 3:

      factor2 = RP/2*sqrt(2*PI)/gammar/tstep;
      p0 = 2*MU*gammar*C;
      break;

    case 4:
    //here we define for Uranium, these numbers can be changed for different types of ions.
    mass = 238.;
    charge = 79.;

      factor2 = charge*RP/mass*sqrt(2*PI)/gammar/tstep;
      p0 = mass*MU*gammar*C;
      break;
    }

    coeff3 = eta/fabs(eta)*charge*E*voltm/(p0*circ);
    coeff4 = wrf/C;
    coeff5 = -eta*C;

    if(isyn==4 || isyn==5 || isyn ==6 || isyn == 7 || isyn == 8) {
      fprintf(prot_inphase, "\n Q_s0=%lf\n eta=%lf \n V_m=%lf \n", sqrt(-coeff3*coeff5*coeff4/omega0/omega0),eta,voltm);
      fprintf(prot_inphase, "Longitudinal matching number: %lf \n", circ/2/PI*fabs(eta)*dp0/sqrt(-coeff3*coeff5*coeff4/omega0/omega0)/sz0);
    fprintf(prot_inphase, "WARNING: If longitudinal matching number is not about 1,\n");
    fprintf(prot_inphase, "the bunch is not matched to the bucket !!!!! \n");
    }
    else if(isyn==1) {
      fprintf(prot_inphase, "Longitudinal matching number: %lf \n", circ/2/PI*fabs(eta)*dp0/nus/sz0);
      fprintf(prot_inphase, "WARNING: If longitudinal matching number is not about 1,\n");
      fprintf(prot_inphase, "the bunch is not matched to the bucket !!!!! \n");
    }

    coeffspch = (0.67 + 2*log(((double)ielsizex+(double)ielsizey)/2))*E*E*npr0/(sqrt(2*PI)*4*PI*EPS0*gammar*gammar*p0);

    fscale2 = factor2/sqrt(2*PI);
    sx0_cd = sqrt(sx0*sx0 + dispx*dispx*dp0*dp0);
    rpipex = sx0_cd*(double)ielsizex;
    rpipey = sy0*(double)ielsizey;

    qp0 = npr0/(double)NPR;   

    switch(elflag){

    case 1:
      qe0 = 4.0*rpipex*rpipey*dlstep*dcloud/(double)NEL;
      break;

    case 2:
      qe0 = PI*rpipex*rpipey*dlstep*dcloud/(double)NEL;
      break;

    case 3:
      qe0 = 8.0*sx0_cd*rpipey*dlstep*dcloud/(double)NEL;
      break;

    case 4:
      qe0 = 2.0*rpipey*8.0*strsig*sx0*dlstep*dcloud/(double)NEL;
      break;     

    case 5:
      qe0 = 4.0*rpipex*rpipey*dlstep*dcloud/(double)NEL;
      break;     
    }

    area = qe0/dlstep/dcloud;
    sx2 = 0.5*sqrt(area*sx0_cd/sy0)/(double)imesize;
    sy2 = sy0/sx0_cd*sx2;

    if (i_phas_adv==0){
    cxa = cos(2*PI*htune/(double)nkick);
    sxa = sin(2*PI*htune/(double)nkick);
    cya = cos(2*PI*vtune/(double)nkick);
    sya = sin(2*PI*vtune/(double)nkick);
    }
    
    if (i_phas_adv==2){
      for (kmain=1; kmain<=nkick; kmain++){
	phadv[kmain]=rand()/(double)RAND_MAX;
	//printf("%12.8e \n", phadv[kmain]);
      }
    }
  


    dtunxmax = C*npr0*E*circ*circ/(4*PI*PI*I0*gammar*gammar*gammar*htune*sx0_cd*sx0_cd*sqrt(2*PI)*sz0);
    dtunymax = C*npr0*E*circ*circ/(4*PI*PI*I0*gammar*gammar*gammar*vtune*sy0*sy0*sqrt(2*PI)*sz0);

    csa = cos(omegas*tstepp);
    ssa = sin(omegas*tstepp);

    nstep = nkick*nturn;

    n_diag=n_diag*nkick;
    switch(ipr_pos){
    case 1:

      wakefac = - E*E/(MU*gammar*C*C);
      break;

    case 2:

      wakefac = - E*E/(ME*gammar*C*C);
      break;

    case 3:

      wakefac = - E*E/(2*MU*gammar*C*C);
      break;

    case 4:

      wakefac = - charge*E*E/(238*MU*gammar*C*C);
      break;
    }

    eel = 1.0;
    epr = 1.0;

    //i_oct = 1; all following settings up to the last one require i_oct=1

    /* SPS octupole setting: radial -0.875 and vertical -0.8 - signs have been
       reversed to check that now space charge should help the decoherence    
    qpxx = 424.9;
    qpxy = -878;
    qpyy = 1155;   
*/
    /* Rogelio: SPS octupole setting: radial -0.875 and vertical -0.8 
    qpxx = -424.9;
    qpxy = 878;
    qpyy = -1155;
    */
    /* Rogelio: SPS octupole setting: radial -0.875 and vertical 1.0
    qpxx = -283.25;
    qpxy = 113.3;
    qpyy = -420;
    */

    /* Rogelio: 27.03.2003 requires i_oct=2, compensates horizontal amplitude
       detuning from sextupolar kick                                           
    qpxx = 2e-4;
    qpxy = 0.;
    qpyy = 0.;
    */

 /* Elena, collimator: 20.10.2003 requires i_oct=3       */
    qpxx = 5e-4;
    qpxy = 0.;
    qpyy = 5e-4;


    /* Coefficients of the magnetic field in the PS: 
       B_x = - bgrad*y, B_y = bdip + bgrad*x
       The signs of the gradients do not affect the induced wake               */

    bdip = 1.2;
    bgrad = 5.2;

    /* Calculates number of sections in which the electron cooler has to be
       subdivided                                                              */

    n_cool = (long)ceil(dl_cool/tstep/C);

}

/******************************************************************************
 *** Subroutine     : build_distrib                                         ***
 *** Effect         : Calculate arbitrary distribution of bunch.            ***
 *** Parameters     : vector containing the quantity whose distribution     ***
 ***                  we want to plot, number of elements of this vector    ***
 *** Gbl var effect :                                                       ***
 *** Constants used :                                                       ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/


void verteil(double data[], unsigned long nele, double distri[], unsigned long ncell,
             double index[])

{ long   j , channel1 , channel2 , numb ;

  double  scaled , mean_data , mean_sqr_data , fraction1 , fraction2 , data_center,
          data_width ;

  mean_data     = 0 ;
  mean_sqr_data = 0 ;

  for (j=0 ; j<nele ; ++j)
         { 
           mean_data     += data[j] ;
           mean_sqr_data += data[j]*data[j] ; }

  mean_data     /= nele ;
  mean_sqr_data /= nele ;

  data_center = mean_data ;
  data_width  = 10.0*sqrt(mean_sqr_data-mean_data*mean_data) ;

  for (j=0 ; j<ncell ; ++j)
      distri[j] = 0.0 ;

  for (j=0 ; j<nele ; ++j)
         { scaled = (data[j]-data_center+data_width/2.0)/data_width*ncell ;
           channel1 = (long) scaled ;
           if (scaled-channel1 > 0.5)
              { fraction1 = 1.5 - scaled + channel1 ;
                channel2  = channel1 + 1 ;
                fraction2 = 1.0 - fraction1 ; }
            else
              { fraction1 = scaled - channel1 + 0.5 ;
                channel2  = channel1 - 1 ;
                fraction2 = 1.0 - fraction1 ; }
           if (channel1 >= 0 && channel1 < ncell)
              distri[channel1] += fraction1 ;
           if (channel2 >= 0 && channel2 < ncell)
              distri[channel2] += fraction2 ; }

  for (j=0 ; j<ncell ; ++j)
    {
      index[j] = data_center - data_width/2.0 + j*data_width/(ncell-1); 
      distri[j] /= data_width/ncell ;
    }

}

/******************************************************************************
 *** Subroutine     : build_distrib2                                        ***
 *** Effect         : Calculate arbitrary distribution.                     ***
 *** Parameters     : vector containing the quantity whose distribution     ***
 ***                  we want to plot, number of elements of this vector    ***
 *** Gbl var effect :                                                       ***
 *** Constants used :                                                       ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

/* specially suitable to plot rectangular distributions, having a sharp edge  */

void verteil2(double data[], unsigned long nele, double distri[], unsigned long ncell,
             double index[])

{ long   j , channel1 , channel2 , numb ;

  double  scaled , mean_data , mean_sqr_data , fraction1 , fraction2 , data_center,
          data_width ;

  mean_data     = 0 ;
  mean_sqr_data = 0 ;

  for (j=0 ; j<nele ; ++j)
         { 
           mean_data     += data[j] ;
           mean_sqr_data += data[j]*data[j] ; }

  mean_data     /= nele ;
  mean_sqr_data /= nele ;

  data_center = mean_data ;
  data_width  = 5.0*sqrt(mean_sqr_data-mean_data*mean_data) ;

  for (j=0 ; j<ncell ; ++j)
      distri[j] = 0.0 ;

  for (j=0 ; j<nele ; ++j)
         { scaled = (data[j]-data_center+data_width/2.0)/data_width*ncell ;
           channel1 = (long) scaled ;
           if (scaled-channel1 > 0.5)
              { fraction1 = 1.5 - scaled + channel1 ;
                channel2  = channel1 + 1 ;
                fraction2 = 1.0 - fraction1 ; }
            else
              { fraction1 = scaled - channel1 + 0.5 ;
                channel2  = channel1 - 1 ;
                fraction2 = 1.0 - fraction1 ; }
           if (channel1 >= 0 && channel1 < ncell)
              distri[channel1] += fraction1 ;
           if (channel2 >= 0 && channel2 < ncell)
              distri[channel2] += fraction2 ; }

  for (j=0 ; j<ncell ; ++j)
    {
      index[j] = data_center - data_width/2.0 + j*data_width/(ncell-1); 
      distri[j] /= data_width/ncell ;
    }

}
  
/******************************************************************************
 *** Subroutine     : parabolic                                             ***
 *** Effect         : Parabolic particle and velocity distribution in       ***
 ***                  longitudinal direction.                               ***
 *** Parameters     : none                                                  ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used :                                                       ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

parabolic ()

{ long   i ;
  double  r1 , r2 , parab_as , parab_b;
  
  srand(time(NULL));
  parab_as = rpipey;
  parab_b = 0.0;

  for (i=0 ; i<NEL ; ++i)
      { r1 = rand() / ((double)RAND_MAX) ;
        r2 = rand() / ((double)RAND_MAX) ;

        xe[i] = rpipex * sqrt(1.0-pow(1.0-r1,2.0/3.0)) * cos(2.0*PI*r2) ;
        ye[i] = (parab_as*sin(2.0*PI*r2) + parab_b*rpipex*cos(2.0*PI*r2)) *
                sqrt(1.0-pow(1.0-r1,2.0/3.0)) ;
        xpe[i] = 0.0;
        ype[i] = 0.0;
      }

}

/******************************************************************************
 *** Subroutine     : initialization                                        ***
 *** Effect         : Initialize particle and velocity distributions.       ***
 *** Parameters     : none                                                  ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used :                                                       ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

initialization ()

{ double xsum, ysum, xpsum, ypsum, xsum2, ysum2,
         u, v, s, indsz[NDC], lgds[NDC], inddp[NDC], lgmm[NDC];

  long i, j, l;  

  xsum = 0.0;
  ysum = 0.0;
  xpsum = 0.0;
  ypsum = 0.0;
  xsum2 = 0.0;
  ysum2 = 0.0;

  if (elflag==1)
     for (i=0; i<NEL; i++)
       { xe[i] = rpipex*2.*(rand ()/(double)RAND_MAX - 0.5);
         ye[i] = rpipey*2.*(rand ()/(double)RAND_MAX - 0.5);
         xpe[i] = 0.0;
         ype[i] = 0.0;
       }

  if (elflag==2)
     for (i=0; i<NEL; i++)
        { do { xe[i] = rpipex*2.*(rand ()/(double)RAND_MAX - 0.5);
               ye[i] = rpipey*2.*(rand ()/(double)RAND_MAX - 0.5);
	     }
          while (xe[i]*xe[i]/rpipex/rpipex + ye[i]*ye[i]/rpipey/rpipey >= 1);
          xpe[i] = 0.0;
          ype[i] = 0.0;
	}

  if(elflag==3)
     for (i=0; i<NEL; i++)
        { if(i % 10==0) 
            xe[i] = rpipex*2.*(rand ()/(double)RAND_MAX - 0.5);
          else 
            xe[i] = 2.*sx0_cd*2.*(rand ()/(double)RAND_MAX - 0.5);

          ye[i] = rpipey*2.*(rand ()/(double)RAND_MAX - 0.5);
          xpe[i] = 0.0;
          ype[i] = 0.0;
	}

  if(elflag==4)
    for (i=0; i<NEL; i++)
        { do { if(i % 10==0) 
                 xe[i] = rpipex*2.*(rand ()/(double)RAND_MAX - 0.5);
               else 
                 if(rand () > 0.5)
                   xe[i] = strpos*sx0+2.*(sx0*strsig)*2.*(rand ()/(double)RAND_MAX - 0.5);
                 else
                   xe[i] = -strpos*sx0+2.*(sx0*strsig)*2.*(rand ()/(double)RAND_MAX - 0.5);
               ye[i] = rpipey*2.*(rand ()/(double)RAND_MAX - 0.5);
	      }
        while (xe[i]*xe[i]/rpipex/rpipex + ye[i]*ye[i]/rpipey/rpipey >= 1);
        xpe[i] = 0.0;
        ype[i] = 0.0;
	}

  if (elflag==5)
     parabolic();

  for (i=0; i<NPR; i++)
    { 
      mark[i]=0;
      do { u = 2.0 * rand () / (double)RAND_MAX - 1.0 ;
            v = 2.0 * rand () / (double)RAND_MAX - 1.0 ;
            s = u*u + v*v ; }
          while (s >= 1.0) ;
 
    xpr[i] = sx0 * u * sqrt(-2.0*log(s)/s);
    xpp[i] = sx0/betax0 * v * sqrt(-2.0*log(s)/s);
       
        do { u = 2.0 * rand () / (double)RAND_MAX - 1.0 ;
             v = 2.0 * rand () / (double)RAND_MAX - 1.0 ;
             s = u*u + v*v ; }
          while (s >= 1.0) ;
 
     ypr[i] = sy0 * u * sqrt(-2.0*log(s)/s);
     ypp[i] = sy0/betay0 * v * sqrt(-2.0*log(s)/s);

     if(isyn==0 || isyn==1 || isyn==2)
       { do { u = 2.0 * rand () / (double)RAND_MAX - 1.0 ;
              v = 2.0 * rand () / (double)RAND_MAX - 1.0 ;
              s = u*u + v*v ; }
           while (s >= 1.0) ;
    
         sz[i] = z0_kick + sz0 * u * sqrt(-2.0*log(s)/s);
         dp[i] = dp0 * v * sqrt(-2.0*log(s)/s);
       }

     if(isyn==3)
        sz[i] = sz0 * 4.0 * (rand () / (double)RAND_MAX - 0.5);

     if(isyn==4 || isyn==5 || isyn==6 || isyn ==7 || isyn == 8)
       { do {
             do { u = 2.0 * rand () / (double)RAND_MAX - 1.0 ;
                  v = 2.0 * rand () / (double)RAND_MAX - 1.0 ;
                  s = u*u + v*v ; }
             while (s >= 1.0) ;

             sz[i] = z0_kick + sz0 * u * sqrt(-2.0*log(s)/s);
             dp[i] = dp0 * v * sqrt(-2.0*log(s)/s);
	       }
	     while (-0.5*coeff5/coeff3*coeff4*dp[i]*dp[i]-cos(coeff4*sz[i])>1);
       }
 
     xsum += xpr[i];
     ysum += ypr[i];
     xpsum += xpp[i];
     ypsum += ypp[i];
    }

  xave = xsum/(double)NPR;
  yave = ysum/(double)NPR;
  xpave = xpsum/(double)NPR;
  ypave = ypsum/(double)NPR;

  contax=0;
  for (l=0; l<NPR; l++)
    { xpr[l] += xkick - xave;
      xpp[l] += -xpave;
      ypr[l] += ykick - yave;
      ypp[l] += -ypave;
    
      if ( ( xpr[l]<(xkick+5e-2*sx0) &&  xpr[l]>(xkick-5e-2*sx0)) && (xpp[l]<1e-1*sx0/betax0 && xpp[l]>-1e-1*sx0/betax0))
      {contax+=1;
      mark[l]=1;
      // fprintf(track_pr,"%d  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e\n",l,xpr[l],xpp[l],ypr[l],ypp[l],sz[l],dp[l]); 
      }
     
      // fprintf(prot_ini,"%d  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e\n",l,xpr[l],xpp[l],ypr[l],ypp[l],sz[l],dp[l]); 

	// fprintf(track_pr,"0 %13.8e %13.8e \n",sz[l],dp[l]);
    }

  // fprintf(prot_ini,"\n");
  // fprintf(prot_ini,"\n");

  printf("%d",contax);

  //fprintf(track_pr,"\n");
  // fprintf(track_pr,"\n");

  //verteil(sz,NPR,lgds,NDC,indsz);
  //verteil(dp,NPR,lgmm,NDC,inddp);
  /*
  for (l=1; l<=NDC; l++)
    fprintf(prot_indistr,"%d  %13.8e  %13.8e\n",l,lgds[l],lgmm[l]);
  */
  /*

  zstep = 4.0*sz0/(double)NBIN;

  for (j=0; j<NBIN; j++)
    { zs[j] = -2.0*sz0 + zstep*0.5 + (double)j*zstep;
      npr[j]=0.0;
      for (l=1; l<=10; l++) 
        npr[j] += npr0/sqrt(2.0*PI)/sz0*(exp(-(zs[j]-zstep/2+zstep/10*(l-1))*(zs[j]-zstep/2+zstep/10*(l-1))/2/sz0/sz0)+exp(-(zs[j]-zstep/2+zstep/10*l)*(zs[j]-zstep/2+zstep/10*l)/2/sz0/sz0))*0.05*zstep;
    }

  */

}

/******************************************************************************
 *** Subroutine     : refresh_el                                            ***
 *** Effect         : Re-initialization of the electrons distribution       ***
 ***                  after one full bunch passage                          ***
 *** Parameters     : none                                                  ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used :                                                       ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

refresh_el ()

{ double xeave, yeave;
  long   i;
 
  xeave = 0.0;
  yeave = 0.0;

  if (elflag==1)
    for (i=0; i<NEL; i++)
       { xe[i] = rpipex*2.*(rand()/(double)RAND_MAX - 0.5);
         ye[i] = rpipey*2.*(rand()/(double)RAND_MAX - 0.5);
         xpe[i] = 0.0;
         ype[i] = 0.0;
         xeave += xe[i];
         yeave += ye[i];
       }

  if (elflag==2)
     for (i=0; i<NEL; i++)
        { do { xe[i] = rpipex*2.*(rand ()/(double)RAND_MAX - 0.5);
               ye[i] = rpipey*2.*(rand ()/(double)RAND_MAX - 0.5);
	     }
          while (xe[i]*xe[i]/rpipex/rpipex + ye[i]*ye[i]/rpipey/rpipey >= 1);
          xpe[i] = 0.0;
          ype[i] = 0.0;
          xeave += xe[i];
          yeave += ye[i];
	}

  if(elflag==3)
     for (i=0; i<NEL; i++)
        { if(i % 10==0) 
            xe[i] = rpipex*2.*(rand ()/(double)RAND_MAX - 0.5);
          else 
            xe[i] = 2.*sx0_cd*2.*(rand ()/(double)RAND_MAX - 0.5);

          ye[i] = rpipey*2.*(rand ()/(double)RAND_MAX - 0.5);
          xpe[i] = 0.0;
          ype[i] = 0.0;
          xeave += xe[i];
          yeave += ye[i];
	}

  if(elflag==4)
    for (i=0; i<NEL; i++)
        { do { if(i % 10==0) 
                 xe[i] = rpipex*2.*(rand ()/(double)RAND_MAX - 0.5);
               else 
                 if(rand () > 0.5)
                   xe[i] = strpos*sx0+2.*(sx0*strsig)*2.*(rand ()/(double)RAND_MAX - 0.5);
                 else
                   xe[i] = -strpos*sx0+2.*(sx0*strsig)*2.*(rand ()/(double)RAND_MAX - 0.5);
               ye[i] = rpipey*2.*(rand ()/(double)RAND_MAX - 0.5);
	      }
        while (xe[i]*xe[i]/rpipex/rpipex + ye[i]*ye[i]/rpipey/rpipey >= 1);
        xpe[i] = 0.0;
        ype[i] = 0.0;
        xeave += xe[i];
        yeave += ye[i];
	}

  if(elflag==5)
    { parabolic();
      for (i=0; i<NEL; i++)
        { xeave += xe[i];
          yeave += ye[i];
	}
    }

  xeave /= (double)NEL;
  yeave /= (double)NEL;

  for (i=0; i<NEL; i++)
     { xe[i] += -xeave;
       ye[i] += -yeave;
     }

}

/******************************************************************************
 *** Subroutine     : new_fraction                                          ***
 *** Effect         : Re-initialization of the fraction of electrons        ***
 ***                  newly generated after one slice                       ***
 *** Parameters     : none                                                  ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used :                                                       ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

new_fraction ()

{ double aux, xeave, yeave, betae;
  long   i,iaux;
 
  xeave = 0.0;
  yeave = 0.0;

  betae = 0.145;

  /* iaux - number of macroelectrons that must be re-initialized at each slice */

  aux = betae*(1+4.*sz0/dl_cool);

  iaux = (long) (NBIN/aux); 
  //printf("%d\n",iaux);

  for (i=0; i<NEL; i++)
     if(i % iaux==0 && i+iaux<NEL)
       { do { xe[i+jmain] = rpipex*2.*(rand()/(double)RAND_MAX - 0.5);
              ye[i+jmain] = rpipey*2.*(rand()/(double)RAND_MAX - 0.5);
            }
         while (xe[i+jmain]*xe[i+jmain]/rpipex/rpipex + ye[i+jmain]*ye[i+jmain]/rpipey/rpipey >= 1);
         xpe[i+jmain] = 0.0;
         ype[i+jmain] = 0.0;
         xeave += xe[i+jmain];
         yeave += ye[i+jmain];
       }

  xeave /= (double)NEL;
  yeave /= (double)NEL;

  for (i=0; i<NEL; i++)
     if(i % iaux==0 && i+iaux<NEL)
       { xe[i+jmain] += -xeave;
         ye[i+jmain] += -yeave;
       }

}

/******************************************************************************
 *** Subroutine     : bining                                                ***
 *** Effect         : sub-divides the bunch into thin slices that           ***
 ***                  interact one by one with the kicking e- cloud.        ***
 *** Parameters     : none                                                  ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used :                                                       ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

bining (double z_cent, double szt)

{ double  xbin[NBIN], x2bin[NBIN], ybin[NBIN], y2bin[NBIN],
          xpbin[NBIN], ypbin[NBIN];
  long    i, index, ncount;
  
  ncount = 0;

  zstep = 2.*(double)nsigmaz*szt/(double)NBIN;
  tstep = zstep/C;
  
  factor1 = RE*C*sqrt(2*PI)/tstep;

  switch(ipr_pos){
    case 1: 
      factor2 = RP*sqrt(2*PI)/gammar/tstep;
      break;

    case 2:
      factor2 = RE*sqrt(2*PI)/gammar/tstep;
      break;

    case 3:
      factor2 = RP/2*sqrt(2*PI)/gammar/tstep;
      break;

    case 4:
      //factor2 = 73./238.*RP*sqrt(2*PI)/gammar/tstep;
      factor2 = charge*RP/mass*sqrt(2*PI)/gammar/tstep;
      break;
    }

  n_cool = (long)ceil(dl_cool/tstep/C);





  for (i=0; i<NBIN; i++)
      zs[i] = z_cent -(double)nsigmaz*szt + zstep*0.5 + (double)i*zstep;

  for (i=0; i<NBIN; i++)
     { iind[i] = 0;
       xbin[i] = 0.0;
       xpbin[i] = 0.0;
       x2bin[i] = 0.0;
       ybin[i] = 0.0;
       ypbin[i] = 0.0;
       y2bin[i] = 0.0;
     }

  for (i=npr_eff; i<NPR; i++)
    { index = (long)floor((sz[i] + (double)nsigmaz*szt - z_cent)/(2.*(double)nsigmaz*szt)*NBIN);
    if (index >= 0 && index < NBIN)
      { indexp[index][iind[index]] = i;
        xbin[index] += xpr[i];
        xpbin[index] += xpp[i];
        x2bin[index] += xpr[i]*xpr[i];
        ybin[index] += ypr[i];
        ypbin[index] += ypp[i];
        y2bin[index] += ypr[i]*ypr[i];
        ++iind[index];
        ++ncount;
      }
    }
  sxs_ave=0.;
  sys_ave=0.;
  for (i=0; i<NBIN; i++)
    { if(iind[i]>NPR1-1)
        { fprintf(stderr, "ERROR: too many particles in one slice !!");
	  exit(2);
	}
      if (iind[i] > 0)
        { xs[i] = xbin[i]/(double)iind[i];
	  ys[i] = ybin[i]/(double)iind[i];
          xps[i] = xpbin[i]/(double)iind[i];
	  yps[i] = ypbin[i]/(double)iind[i];
          npr[i] = (double)iind[i]/(double)NPR*npr0;
          if (iind[i] > 1)    
            { sxs[i] = sqrt(x2bin[i]/(double)iind[i] - xs[i]*xs[i]);
              sys[i] = sqrt(y2bin[i]/(double)iind[i] - ys[i]*ys[i]);
            }
          else
	    { sxs[i] = sx0_cd;
      	      sys[i] = sy0;
            }
	}
      else 
	{ xs[i] = 0.0;
	  ys[i] = 0.0;
          xps[i] = 0.0;
	  yps[i] = 0.0;
          npr[i] = 0.0;
          sxs[i] = sx0_cd;
          sys[i] = sy0;
	}
      sxs_ave+=sxs[i];
      sys_ave+=sys[i];
    }
  sxs_ave/=(double)NBIN;
  sys_ave/=(double)NBIN;

  bin_frac = (double)ncount/(double)(NPR - npr_eff);

}

/******************************************************************************
 *** Subroutine     : wake_function                                         ***
 *** Effect         : Function that gives the wake function at a given      ***
 ***                  location z < 0 (z is 'pos')                           ***
 *** Parameters     : pos                                                   ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used : none                                                  ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

double wake_func (double pos)

{ double field;

  field = (zetat*1e6*omegar*omegar/merit/omegabar)*exp(alpha*pos/C)*sin(omegabar*pos/C);
  return field;

}

/******************************************************************************
 *** Subroutine     : wake_function2                                        ***
 *** Effect         : Function that gives the wake function at a given      ***
 ***                  location z < 0 (z is 'pos') for a resistive wall      ***
 *** Parameters     : pos                                                   ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used : none                                                  ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

double wake_reswall (double pos)

{ double sigma, piper, length, field ;

  //sigma = 5.4e13;
  sigma = 5.4e17;
  piper=rpipex;
  length=circ;
  /* copper conductivity in CGS [1/s] */

  if (i_pipe==6) 
    {sigma=0.643e15;
       //carbon resistivity is rho=14e-6 Ohm.m, so this is the conductivity in cgs units
    piper=2e-3; 
        //size of the collimator
    length=1;
    }

  field = -1/PI/piper/piper/piper/2/PI/EPS0*sqrt(C/sigma)*1/sqrt(-pos)*length;


  return field;

}
   
/******************************************************************************
 *** Subroutine     : longitudinal wake_function                            ***
 *** Effect         : Function that gives the z-wake function at a given    ***
 ***                  location z < 0 (z is 'pos') - single bunch in cavity  ***
 *** Parameters     : pos                                                   ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used : none                                                  ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

double wake_funcz (double pos)

{ double field ;

  field = (rshz*1e6*2*alphaz)*exp(alphaz*pos/C)*(cos(omegabarz*pos/C) + alphaz/omegabarz*sin(omegabarz*pos/C));
  return field;

}
   
/******************************************************************************
 *** Subroutine     : open_files                                            ***
 *** Effect         : Open files for storing calculation data.              ***
 *** Parameters     : none                                                  ***
 *** Gbl var   used :                                                       ***
 *** Gbl var effect :                                                       ***
 *** Constants used : none                                                  ***
 *** Subrout.  used : none                                                  ***
 ******************************************************************************/

open_files ()

{ char  bpr_filename[80]   ,                /* bunch properties            */ 
        headtail_fname[80] ,                /* head tail distribution      */ 
        prphase_fname[80]  ,                /* proton phase space          */
        pht_filename[80]   ,
        prot_filename[80]  ,
    samp_filename[80]   ,
    pini_filename[80]   ,
        track_filename[80] ;         /* coordinates of 1000 particles each turn */
 strcpy (track_filename,filename) ;
  strcat (track_filename,"_trk.dat") ;
  track_pr = fopen(track_filename,"w");

  strcpy (bpr_filename,filename) ;
  strcat (bpr_filename,"_prt.dat") ;
  bunch_pr = fopen(bpr_filename,"w");

  strcpy (headtail_fname,filename) ;
  strcat (headtail_fname,"_hdtl.dat") ;
  headtail_pr = fopen(headtail_fname,"w");

  strcpy (prphase_fname,filename) ;
  strcat (prphase_fname,"_prb.dat") ;
  prot_phase = fopen(prphase_fname,"w");

  strcpy (pht_filename,filename) ;
  strcat (pht_filename,"_inph.dat") ;
  prot_inphase = fopen(pht_filename,"w");

  strcpy (pini_filename,filename) ;
  strcat (pini_filename,"_pini.dat") ;
  prot_ini = fopen(pini_filename,"w");


  strcpy (samp_filename,filename) ;
  strcat (samp_filename,"_sample.dat") ;
  samp_prot = fopen(samp_filename,"w");

  strcpy (prot_filename,filename) ;
  strcat (prot_filename,"_bunchds.dat") ;
  prot_distr = fopen(prot_filename,"w");

}

/******************************************************************************
 ****                             PIC module                               ****
 ******************************************************************************/

void*
xmalloc (size_t size) {
  void* tmp;
  tmp=malloc(size);
  if (tmp!=NULL) {
    return tmp;
  }
  fprintf(stderr,"ERROR: not enough memory\n");
  exit(1);
}

/*
  Distributes the beam particles for field calculation
*/

void particles_distribute(GRID *grid,double x[],double y[],int n, int i_which,
                          double n_macro)
{
  int i,j,i1,i2,n_h,j1,j2,j3,j4;
  double delta_x_i,delta_y_i,offset_x,offset_y;
  double h_x,h_y;
  double *rho,tmp;
  int n_x,n_y;
  int distribute_in=0,distribute_out=0;

  n_x=grid->n_cell_x;
  n_y=grid->n_cell_y;
  if (i_which==1) {
    rho=grid->rho1;
  }
  else {
    rho=grid->rho2;
  }
  delta_x_i=1.0/grid->delta_x;
  delta_y_i=1.0/grid->delta_y;
  offset_x=grid->offset_x;
  offset_y=grid->offset_y;
  for (i=0;i<n_x*n_y;i++){
    rho[i]=0.0;
  }
  switch (grid->distribution) {
  case 1:
    for (i=0;i<n;i++){
      i1=(int)floor(x[i]*delta_x_i+offset_x);
      i2=(int)floor(y[i]*delta_y_i+offset_y);
      if ((i1>=0)&&(i1<n_x)&&(i2>=0)&&(i2<n_y)){
        rho[i1*n_y+i2] += n_macro;
        distribute_in++;
      }
      else{
        distribute_out++;
      }
    }
    break;
  case 2:
    for (i=0;i<n;i++){
      h_x=(x[i]*delta_x_i+offset_x-0.5);
      i1=(int)floor(h_x);
      h_x -= (float)i1;
      h_y=(y[i]*delta_y_i+offset_y-0.5);
      i2=(int)floor(h_y);
      h_y -= (float)i2;
      if ((i1>=0)&&(i1<n_x-1)&&(i2>=0)&&(i2<n_y-1)){
        j=i1*n_y+i2;
        rho[j] += (1.0-h_x)*(1.0-h_y)*n_macro;
        j=(i1+1)*n_y+i2;
        rho[j] += h_x*(1.0-h_y)*n_macro;
        j=i1*n_y+i2+1;
        rho[j] += (1.0-h_x)*h_y*n_macro;
        j=(i1+1)*n_y+i2+1;
        rho[j] += h_x*h_y*n_macro;
        distribute_in++;
      }
      else{
        distribute_out++;
      }
    }
    break;
  }
}

/**********************************************/
/* Routines for the calculation of the fields */
/**********************************************/

/* This routine calculates the potentials on the outer boundary of the grid
   by direct calculation. */

void foldfronts (double *rho,double *dist,double *phi,int n_x,int n_y)
{
  double s;
  int i1,i2,i3,i4,j;
  float eps=1e-5;

  i2=0;
  for (i1=0;i1<n_x;i1++)
    {
      s=0.0;
      for (i3=0;i3<n_x;i3++)
        {
          for (i4=0;i4<n_y;i4++)
            {
              s+=rho[i3*n_y+i4]
                *dist[abs(i1-i3)*n_y+abs(i2-i4)];
            }
        }
      phi[i1*n_y+i2]=s;
    }
  i2=n_y-1;
  for (i1=0;i1<n_x;i1++)
    {
      s=0.0;
      for (i3=0;i3<n_x;i3++)
        {
          for (i4=0;i4<n_y;i4++)
            {
              s+=rho[i3*n_y+i4]
                *dist[abs(i1-i3)*n_y+abs(i2-i4)];
            }
        }
      phi[i1*n_y+i2]=s;
    }
  i1=0;
  for (i2=1;i2<n_y-1;i2++)
    {
      s=0.0;
      for (i3=0;i3<n_x;i3++)
        {
          for (i4=0;i4<n_y;i4++)
            {
              s+=rho[i3*n_y+i4]
                *dist[abs(i1-i3)*n_y+abs(i2-i4)];
            }
        }
      phi[i1*n_y+i2]=s;
    }
  i1=n_x-1;
  for (i2=1;i2<n_y-1;i2++)
    {
      s=0.0;
      for (i3=0;i3<n_x;i3++)
        {
          for (i4=0;i4<n_y;i4++)
            {
              s+=rho[i3*n_y+i4]
                *dist[abs(i1-i3)*n_y+abs(i2-i4)];
            }
        }
      phi[i1*n_y+i2]=s;
    }
}

#include "fourtrans2.c"

/* Fast fourier transformation routine used by foldfft */

void fourtrans (double* data,int nn[],int isign)
{
  /* System generated locals */
  int i__1, i__2, i__3, i__4, i__5, i__6;
  double d__1;
  
  /* Builtin functions */
  //  double sin();
  
  /* Local variables */
  int idim, ibit, nrem, ntot, i2rev, i3rev, n;
  double theta, tempi, tempr;
  int nprev, i2;
  double wtemp;
  int i1, i3, k1, k2;
  double wi, wr;
  int ip1, ip2, ip3;
  double wpi, wpr;
  int ifp1, ifp2;
  int ndim=2;
  /* Parameter adjustments */
  --nn;
  --data;
  
  /* Function Body */
  ntot = 1;
  i__1 = ndim;
  for (idim = 1; idim <= i__1; ++idim)
    {
      ntot *= nn[idim];
    }
  nprev = 1;
  i__1 = ndim;
  for (idim = 1; idim <= i__1; ++idim) {
    n = nn[idim];
    nrem = ntot / (n * nprev);
    ip1 = nprev << 1;
    ip2 = ip1 * n;
    ip3 = ip2 * nrem;
    i2rev = 1;
    i__2 = ip2;
    i__3 = ip1;
    for (i2 = 1; i__3 < 0 ? i2 >= i__2 : i2 <= i__2; i2 += i__3) {
      if (i2 < i2rev) {
        i__4 = i2 + ip1 - 2;
        for (i1 = i2; i1 <= i__4; i1 += 2) {
          i__5 = ip3;
          i__6 = ip2;
          for (i3 = i1; i__6 < 0 ? i3 >= i__5 : i3 <= i__5; i3 += 
               i__6) {
            i3rev = i2rev + i3 - i2;
            tempr = data[i3];
            tempi = data[i3 + 1];
                        data[i3] = data[i3rev];
            data[i3 + 1] = data[i3rev + 1];
            data[i3rev] = tempr;
            data[i3rev + 1] = tempi;
          }
        }
      }
      ibit = ip2 / 2;
    L1:
      if (ibit >= ip1 && i2rev > ibit) {
        i2rev -= ibit;
        ibit /= 2;
        goto L1;
      }
      i2rev += ibit;
    }
    ifp1 = ip1;
  L2:
    if (ifp1 < ip2) {
      ifp2 = ifp1 << 1;
      theta = isign * 6.28318530717959 / (ifp2 / ip1);
      /* Computing 2nd power */
      d__1 = sin(theta * .5);
      wpr = d__1 * d__1 * -2.;
      wpi = sin(theta);
      wr = 1.;
      wi = 0.;
      i__3 = ifp1;
      i__2 = ip1;
      for (i3 = 1; i__2 < 0 ? i3 >= i__3 : i3 <= i__3; i3 += i__2) {
        i__4 = i3 + ip1 - 2;
        for (i1 = i3; i1 <= i__4; i1 += 2) {
          i__6 = ip3;
                    i__5 = ifp2;
          for (i2 = i1; i__5 < 0 ? i2 >= i__6 : i2 <= i__6; i2 += 
               i__5) {
            k1 = i2;
            k2 = k1 + ifp1;
            tempr = wr * data[k2] - wi * data[k2 + 1];
            tempi = wr * data[k2 + 1] + wi * data[k2];
            data[k2] = data[k1] - tempr;
            data[k2 + 1] = data[k1 + 1] - tempi;
            data[k1] += tempr;
            data[k1 + 1] += tempi;
          }
        }
        wtemp = wr;
        wr = wr * wpr - wi * wpi + wr;
        wi = wi * wpr + wtemp * wpi + wi;
      }
      ifp1 = ifp2;
      goto L2;
    }
    nprev = n * nprev;
  }
  return;
}

/* This routine calculates the potentials with the help of the fast fourier
   transform */

void fold_fft(double *rho1,double *rho2,double *dist,double *phi1,
              double *phi2,double temp[],int n_x,int n_y)
{
  FILE *datei;
  double help;
  int i1,i2,i,j,j0;
  int nn[2];
  double eps=1e-5;

  /* return no field */

  /* fill array with charge */

  for (i=0;i<n_x*n_y*8;i++) {
    temp[i]=0.0;
  }
  for (i1=0;i1<n_x;i1++) {
    for (i2=0;i2<n_y;i2++) {
      j=2*(i1*2*n_y+i2);
      j0=i1*n_y+i2;
      temp[j]=rho1[j0];
      temp[j+1]=rho2[j0];
    }
  }
  
  nn[0]=2*n_y;
  nn[1]=2*n_x;
  i1=2;
  i2=1;
  //fourtrans(temp,nn,i2);
  fourtrans3(temp,nn,i2);
  for (i=0;i<n_x*n_y*4;i++) {
    j=2*i;
    help=temp[j]*dist[j]-temp[j+1]*dist[j+1];
    temp[j+1]=temp[j+1]*dist[j]+temp[j]*dist[j+1];
    temp[j]=help;
  }
  i2=-1;
  //fourtrans(temp,nn,i2);
  fourtrans3(temp,nn,i2);
  for (i1=0;i1<n_x;i1++) {
    for (i2=0;i2<n_y;i2++) {
      phi1[i1*n_y+i2]=temp[2*(i1*2*n_y+i2)]/((double)4*n_x*n_y);
      phi2[i1*n_y+i2]=temp[2*(i1*2*n_y+i2)+1]/((double)4*n_x*n_y);
    }
  }
}

void init_sor (double *a,double *b,double *c,double *d,
double *e,GRID grid)
{
  double a0,b0,c0,d0,e0,factor;
  int i;

  factor=1.0;
  a0=factor*(grid.delta_y/grid.delta_x);
  b0=a0;
  c0=factor*(grid.delta_x/grid.delta_y);
  d0=c0;
  e0=-2.0*(a0+c0);
  for (i=0;i<grid.n_cell_x*grid.n_cell_y;i++)
    {
      a[i]=a0;
      b[i]=b0;
      c[i]=c0;
      d[i]=d0;
      e[i]=e0;
    }
}

void sor (double *a,double *b,double *c,double *d,double *e,
          double *rho, double *phi,int n_x,int n_y)
{
  const int MAXIT=1000;
  const double eps=1e-5;
  double anormf=0.0,anorm;
  double omega,resid;
  int i1,i2,n,j;
  float rjac,delta_x=1000.0,delta_y=64.0;

  rjac=(delta_y*delta_y*cos(PI/(float)n_x)+delta_x*delta_x*cos(PI/(float)n_y))
    /(delta_x*delta_x+delta_y*delta_y);
  for (i1=1;i1<n_x-1;i1++)
    {
      for (i2=1;i2<n_y-1;i2++)
        {
          anormf += fabs(rho[i1*n_y+i2]);
        }
    }
  omega=1.0;
  for (n=0;n<MAXIT;n++)
    {
      anorm=0.0;
      for (i1=1;i1<n_x-1;i1++)
        {
          for (i2=1;i2<n_y-1;i2++)
            {
              if (((i1+i2) % 2)==(n % 2))
                {
                  j=i1*n_y+i2;
                  resid=a[j]*phi[j+n_y]+b[j]*phi[j-n_y]+c[j]*phi[j+1]
                    +d[j]*phi[j-1]+e[j]*phi[j]-rho[j];
                  anorm += fabs(resid);
                  phi[j] -= omega*resid/e[j];
                }
            }
        }
      if (n==0)
        {
          omega=1.0/(1.0-0.5*rjac*rjac);
        }
      else
        {
          omega=1.0/(1.0-0.25*rjac*rjac*omega);
        }
      if ((n>0)&&(anorm<eps*anormf))
        {
          return;
        }
    }
  printf ("Mist");
  scanf ("%d",&i1);
}

/* Routine to calculate the parameters necessary for the iterative method of
   the potential calculation sor2 */

void init_sor2 (GRID grid,double *parameter)
{
  double factor;
  factor=1.0;
  parameter[0]=factor*(grid.delta_y/grid.delta_x);
  parameter[1]=parameter[0];
  parameter[2]=factor*(grid.delta_x/grid.delta_y);
  parameter[3]=parameter[2];
  parameter[4]=-2.0*(parameter[0]+parameter[2]);
  parameter[5]=(grid.delta_y*grid.delta_y*cos(PI/(float)grid.n_cell_x)
                +grid.delta_x*grid.delta_x*cos(PI/(float)grid.n_cell_y))
                        /(grid.delta_x*grid.delta_x+grid.delta_y*grid.delta_y);
}

/* Routine to calculate the potentials with an iterative method */

void sor2 (double *rho,double *phi,int n_x,int n_y,double *parameter)
{
  const int MAXIT=1000;
  const double eps=1e-5;
  double anormf=0.0,anorm;
  double omega,resid,e_inv;
  int i1,i2,n,j;
  double a,b,c,d,e,rjac;

  a=parameter[0];
  b=parameter[1];
  c=parameter[2];
  d=parameter[3];
  e=parameter[4];
  rjac=parameter[5];
  for (i1=1;i1<n_x-1;i1++)
    {
      for (i2=1;i2<n_y-1;i2++)
        {
          anormf += fabs(rho[i1*n_y+i2]);
        }
    }
  omega=1.0;
  e_inv=1.0/e;
  for (n=0;n<MAXIT;n++)
    {
      anorm=0.0;
      for (i1=1;i1<n_x-1;i1++)
        {
          for (i2=1;i2<n_y-1;i2++)
            {
              if (((i1+i2) % 2)==(n % 2))
                {
                  j=i1*n_y+i2;
                  resid=a*phi[j+n_y]+b*phi[j-n_y]+c*phi[j+1]
                    +d*phi[j-1]+e*phi[j]-rho[j];
                  anorm += fabs(resid);
                  phi[j] -= omega*resid*e_inv;
                }
            }
        }
      if (n==0)
        {
          omega=1.0/(1.0-0.5*rjac*rjac);
        }
      else
        {
          omega=1.0/(1.0-0.25*rjac*rjac*omega);
        }
      if ((n>0)&&(anorm<eps*anormf))
        {
          return;
        }
    }
  printf ("Warning: too many iterations in function sor2\n");
}

double
f_potential_2(double x0,double y0,double dx,double dy)
{
  double x,y,sum;
  x=x0+dx;
  y=y0+dy;
  sum=x*y*(log(x*x+y*y)-3.0)
      +x*x*atan(y/x)+y*y*atan(x/y);
  x=x0+dx;
  y=y0-dy;
  sum-=x*y*(log(x*x+y*y)-3.0)
      +x*x*atan(y/x)+y*y*atan(x/y);
  x=x0-dx;
  y=y0+dy;
  sum-=x*y*(log(x*x+y*y)-3.0)
      +x*x*atan(y/x)+y*y*atan(x/y);
  x=x0-dx;
  y=y0-dy;
  sum+=x*y*(log(x*x+y*y)-3.0)
      +x*x*atan(y/x)+y*y*atan(x/y);
  return sum;
}

/* This routine initialises the grid for the calculation of the potentials.
   what_grid decides what distribution will be used, what_grid=2 should be
   prefered. */

void
grid_init_phys (GRID *grid,float cut_x,float cut_y)
{
   FILE *datei;
   int i1,i2,j,j0;
   double factor,phi0;
   double x0,y0;
   int n_x,n_y,n_z;
   int nn[2];

   n_x=grid->n_cell_x;
   n_y=grid->n_cell_y;
   grid->min_x=-((float)n_x-2)/((float)n_x)*cut_x;
   grid->max_x=((float)n_x-2)/((float)n_x)*cut_x;
   grid->min_y=-((float)n_y-2)/((float)n_y)*cut_y;
   grid->max_y=((float)n_y-2)/((float)n_y)*cut_y;
   grid->offset_x=0.5*(float)n_x;
   grid->offset_y=0.5*(float)n_y;
   grid->delta_x=2.0*cut_x/((double)n_x);
   grid->delta_y=2.0*cut_y/((double)n_y);
   factor=1.0/(grid->delta_x*grid->delta_y);
   switch (grid->integration_method) {
   case 1:
   case 3:
     phi0=factor*
       f_potential_2(0.0,0.0,0.5*grid->delta_x,0.5*grid->delta_y);
     for (i1=0;i1<grid->n_cell_x;i1++) {
       for (i2=0;i2<grid->n_cell_y;i2++) {
         j=i1*grid->n_cell_y+i2;
         x0=(double)i1;
         y0=(double)i2;
         grid->dist[j]=factor*
           f_potential_2(x0*grid->delta_x,y0*grid->delta_y,
                         0.5*grid->delta_x,0.5*grid->delta_y)
           -phi0;
       }
     }
     break;
   case 2:
     phi0=factor*
       f_potential_2(0.0,0.0,0.5*grid->delta_x,0.5*grid->delta_y);
     for (i1=0;i1<n_x*n_y*8;i1++) {
       grid->dist[i1]=0.0;
     }
     for (i1=0;i1<n_x;i1++) {
       for (i2=0;i2<n_y;i2++) {
         j0=2*(i1*2*n_y+i2);
         x0=(double)i1;
         y0=(double)i2;
         grid->dist[j0]=factor*
           f_potential_2(x0*grid->delta_x,y0*grid->delta_y,
                         0.5*grid->delta_x,0.5*grid->delta_y)
           -phi0;
         if (i2>=1) {
           grid->dist[2*((i1+1)*2*n_y-i2)]=grid->dist[j0];
           if (i1>=1) {
             grid->dist[2*(2*n_y*(2*n_x-i1)+i2)]=grid->dist[j0];
             grid->dist[2*(2*n_y*(2*n_x-i1)+2*n_y-i2)]=grid->dist[j0];
           }
         }
         else {
           if (i1>=1) {
             grid->dist[2*(2*n_y*(2*n_x-i1)+i2)]=grid->dist[j0];
           }
         }
       }
     }
     nn[0]=2*n_y;
     nn[1]=2*n_x;
     i1=2;
     i2=1;
     //fourtrans(grid->dist,nn,i2);
     fourtrans2(grid->dist,nn,i2);
     break;
   }
}

GRID*
grid_init_comp (int n_x,int n_y,int integration_method)
{
  GRID *grid;
  int i1,i2,j;
  
  grid=(GRID*)xmalloc(sizeof(GRID));
  grid->n_cell_x=n_x;
  grid->n_cell_y=n_y;

  grid->integration_method=integration_method;
  grid->distribution=2;
  grid->interpolation=2;

  grid->rho1=(double*)xmalloc(sizeof(double)*n_x*n_y);
  grid->rho2=(double*)xmalloc(sizeof(double)*n_x*n_y);
  grid->phi1=(double*)xmalloc(sizeof(double)*n_x*n_y);
  grid->phi2=(double*)xmalloc(sizeof(double)*n_x*n_y);
  if (integration_method==2) {
    grid->dist=(double*)xmalloc(sizeof(double)*n_x*n_y*8);
    grid->temp=(double*)xmalloc(sizeof(double)*n_x*n_y*8);
  }
  else {
    grid->dist=(double*)xmalloc(sizeof(double)*n_x*n_y);
  }
  return grid;
}

/* This routine folds the charge distribution and the greensfunction of
a grid to get the potentials. */

void foldfields (double *rho,double *dist,double *phi,int n_x,int n_y)
{
  double s;
  int i1,i2,i3,i4,j;
  int field_typ=1;
  float eps=1e-5;

  for (i1=0;i1<n_x;i1++) {
    for (i2=0;i2<n_y;i2++) {
      s=0.0;
      for (i3=0;i3<n_x;i3++) {
        for (i4=0;i4<n_y;i4++) {
          s+=rho[i3*n_y+i4]*dist[abs(i1-i3)*n_y+abs(i2-i4)];
        }
      }
      phi[i1*n_y+i2]=s;
    }
  }
}

void
field_calculate(GRID *grid)
{
  switch (grid->integration_method) {
  case 1:
    foldfields(grid->rho1,grid->dist,grid->phi1,grid->n_cell_x,grid->n_cell_y);
    foldfields(grid->rho2,grid->dist,grid->phi2,grid->n_cell_x,grid->n_cell_y);
    break;
  case 2:
    fold_fft(grid->rho1,grid->rho2,grid->dist,grid->phi1,grid->phi2,
             grid->temp,grid->n_cell_x,grid->n_cell_y);
    break;
  }
}

void particles_move(GRID *grid,double energy,double ics0[],
                    double yps0[],double icsp[],double ypsp[],
                    int nn,int i_which,double step,double scale)
{
  static float esum=0.0;
  static int sum_photon=0;
  static long int n_low=0;
  int i,i1,i2,l,k;
  
int elconta;

  double ax,ay;
  double phi1_x,phi2_x,phi3_x,phi1_y,phi2_y,phi3_y,h_x,h_y;
  int n_x,n_y;
  register int j;
  register double h,h_p;
  double *phi;
  double delta_x_i,delta_y_i,offset_x,offset_y,delta_x,delta_y;
  double xx,yy,xxp,yyp;
  double min_x,max_x,min_y,max_y,e_tmp;
  double k11,k12,k13,k14,k21,k22,k23,k24,k31,k32,k33,k34,k41,k42,k43,k44;
  double velprx, velpry, bux, buy, bmod;
  double p;

  elconta=0;

  min_x=grid->min_x;
  max_x=grid->max_x;
  min_y=grid->min_y;
  max_y=grid->max_y;
  delta_x=grid->delta_x;
  delta_y=grid->delta_y;
  delta_x_i=1.0/grid->delta_x;
  delta_y_i=1.0/grid->delta_y;

  offset_x=grid->offset_x;
  offset_y=grid->offset_y;
  n_x=grid->n_cell_x;
  n_y=grid->n_cell_y;

  if (i_which==1){
    phi=grid->phi2;
  }
  else{
    phi=grid->phi1;
  }
  switch (grid->interpolation){
  case 1:
    for (i=0;i<nn;i++){
      i1=(int)floor(ics0[i]*delta_x_i+offset_x);
      i2=(int)floor(yps0[i]*delta_y_i+offset_y);
      if ((i1<n_x-1)&&(i1>0)&&(i2<n_y-1)&&(i2>0)){
        j=i1*n_y+i2;
        ax = (phi[j+n_y]
              -phi[j-n_y])*step
          /(2.0*delta_x*energy);
        ay = (phi[j+1]
              -phi[j-1])*step
          /(2.0*delta_y*energy);
        icsp[i] -= ax*scale;
        ypsp[i] -= ay*scale;
      }
      else {
        fprintf(stderr,"WARNING: particle is not on the grid\n");
      }
      ics0[i] += icsp[i]*step;
      yps0[i] += ypsp[i]*step;
    }
    break;
    case 2:
      for (i=0;i<nn;i++){
        if(i_which==2 && i_dip!=2) {
          ics0[i] += icsp[i]*step;
          yps0[i] += ypsp[i]*step;
          if(elflag==1 || elflag==3 || elflag==4)
            { if (fabs(ics0[i])>=rpipex || fabs(yps0[i])>=rpipey)
	        { do { if(i_dip==1)
                         icsp[i] = 0.0;
                       else
                         icsp[i] = 1.3e6*rand () / (double)RAND_MAX  ;
                       ypsp[i] = 1.3e6*rand () / (double)RAND_MAX  ;
		     }
                  while (sqrt(icsp[i]*icsp[i]+ypsp[i]*ypsp[i]) >= 1.3e6) ;
		  if(fabs(ics0[i])>=rpipex)
                     ics0[i] = fabs(ics0[i])/ics0[i]*rpipex;
                  if(fabs(yps0[i])>=rpipey)
		     yps0[i] = fabs(yps0[i])/yps0[i]*rpipey;
	          icsp[i] *= -fabs(ics0[i])/ics0[i];
	          ypsp[i] *= -fabs(yps0[i])/yps0[i];
		}
	    }
          if(elflag==2)
            { if (ics0[i]*ics0[i]/rpipex/rpipex+yps0[i]*yps0[i]/rpipey/rpipey>=1.)
	        { do { if(i_dip==1)
                          icsp[i] = 0.0;
                       else
                          icsp[i] = 1.3e6*rand () / (double)RAND_MAX  ;
                       ypsp[i] = 1.3e6*rand () / (double)RAND_MAX  ;
                      }
                  while (sqrt(icsp[i]*icsp[i]+ypsp[i]*ypsp[i]) >= 1.3e6) ;
		  ics0[i] = rpipex*cos(atan2(yps0[i],ics0[i]));
		  yps0[i] = rpipey*sin(atan2(yps0[i],ics0[i]));
	          icsp[i] *= -fabs(ics0[i])/ics0[i];
	          ypsp[i] *= -fabs(yps0[i])/yps0[i];
		}
	    }
	}
        xx=ics0[i];
        yy=yps0[i];
        xxp=icsp[i];
        yyp=ypsp[i];
        if ((xx>min_x)&&(xx<max_x)&&(yy>min_y)&&(yy<max_y)){
          h_x=xx*delta_x_i+offset_x;
          h_y=yy*delta_y_i+offset_y;
          i1=(int)floor(h_x);
          h=h_y-0.5;
          i2=(int)floor(h);
          h -= i2;
          j=i1*n_y+i2;
          h_p=1.0-h;
          phi1_y=h*phi[j+n_y+1]
            +h_p*phi[j+n_y];
          phi2_y=h*phi[j+1]
            +h_p*phi[j];
          phi3_y=h*phi[j-n_y+1]
            +h_p*phi[j-n_y];
          i2=(int)floor(h_y);
          h=h_x-0.5;
          i1=(int)floor(h);
          h -= i1;
          h_p=1.0-h;
          j=i1*n_y+i2;
          phi1_x=h*phi[j+n_y+1]
            +h_p*phi[j+1];
          phi2_x=h*phi[j+n_y]
            +h_p*phi[j];
          phi3_x=h*phi[j+n_y-1]
            +h_p*phi[j-1];
          h_x -= floor(h_x);
          h_y -= floor(h_y);
          ax = (h_x*(phi1_y-phi2_y)+(1.0-h_x)*(phi2_y-phi3_y))
            /(delta_x*energy);
          ay = (h_y*(phi1_x-phi2_x)+(1.0-h_y)*(phi2_x-phi3_x))
            /(delta_y*energy);
	  /*        if(i_which==1)
            fprintf(campo_dat, "%13.8e  %13.8e\n",sqrt(x*x+y*y),sqrt(ax*ax+ay*ay));
          else
            fprintf(campo2_dat, "%13.8e  %13.8e\n",sqrt(x*x+y*y),sqrt(ax*ax+ay*ay));
	  */ 

          switch(i_dip){

            case 0:

                 icsp[i] -= ax*step*scale;
                 ypsp[i] -= ay*step*scale;
                 break;	 

	    case 1:

                 icsp[i] -= 0.0;
                 ypsp[i] -= ay*step*scale;
                 break;

	    case 2:

                 if(i_which==2)
                  { k11 = xxp*step;
                    k21 = (-ax*scale + E/ELMS*bsol*yyp)*step;
                    k31 = yyp*step;
                    k41 = (-ay*scale - E/ELMS*bsol*xxp)*step;

                    k12 = (xxp+k21/2)*step;
                    k22 = (-ax*scale + E/ELMS*bsol*(yyp+k41/2))*step;
                    k32 = (yyp+k41/2)*step;
                    k42 = (-ay*scale - E/ELMS*bsol*(xxp+k21/2))*step;

                    k13 = (xxp+k22/2)*step;
                    k23 = (-ax*scale + E/ELMS*bsol*(yyp+k42/2))*step;
                    k33 = (yyp+k42/2)*step;
                    k43 = (-ay*scale - E/ELMS*bsol*(xxp+k22/2))*step;

                    k14 = (xxp+k23)*step;
                    k24 = (-ax*scale + E/ELMS*bsol*(yyp+k43))*step;
                    k34 = (yyp+k43)*step;
                    k44 = (-ay*scale - E/ELMS*bsol*(xxp+k23))*step;

                    ics0[i] += k11/6 + k12/3 + k13/3 + k14/6;
                    icsp[i] += k21/6 + k22/3 + k23/3 + k24/6;
                    yps0[i] += k31/6 + k32/3 + k33/3 + k34/6;
                    ypsp[i] += k41/6 + k42/3 + k43/3 + k44/6;

		  }

                 if(i_which==1)
                  { icsp[i] -= ax*scale*step;
   	            ypsp[i] -= ay*scale*step;
		  }
                 break;

	    case 3:

	         if(i_which==2)
                  { velprx = icsp[i] - ax*step*scale;
                    velpry = ypsp[i] - ay*step*scale;
                    bux = bgrad*yps0[i];
                    buy = bdip - bgrad*ics0[i];
                    bmod = sqrt(bux*bux+buy*buy);
                    bux *= fabs(velprx)/velprx/bmod;
                    buy *= fabs(velpry)/velpry/bmod;
                    icsp[i] = (bux*velprx + buy*velpry)*bux;
                    ypsp[i] = (bux*velprx + buy*velpry)*buy;
		  }
                 if(i_which==1)
                  { icsp[i] -= ax*scale*step;
   	            ypsp[i] -= ay*scale*step;
		  }
                 break;

	  }
	}

        else{
          //fprintf(stderr,"WARNING: particle is not on the grid\n");
          if (i_which==2)
            //fprintf(stderr,"WARNING: it's an electron \n"); 
	    elconta+=1;
	  else
            fprintf(stderr,"WARNING: it's a proton  (%13.8e,%13.8e,%d,%d)\n", xx,yy,i,nn); 
        }
      }
      break;
    }
  // fprintf(stderr,"nturn= %d electr= %d \n", it, elconta);
}

void nrerror(char error_text[])

{
  fprintf(stderr,"ERROR...\n");
  fprintf(stderr,"%s\n",error_text);
  fprintf(stderr,"...now exiting...\n");
  exit(1);
}

double *dvector(long n1, long nh)
     /* allocate a double vector with subscript range v[n1..nh] */
{
  double *v;
  
  v=(double *)malloc((size_t) ((nh-n1+1+NR_END)*sizeof(double)));
  if (!v) nrerror("allocation failure in dvector()");
  return v-n1+NR_END;
}

void free_dvector(double *v, long n1, long nh)
     /* free a double vector allocated with dvector() */
{
  free((FREE_ARG) (v+n1-NR_END));
}
void
field_poissonbc(GRID *grid)

{
  double    *rho11,
            *rho22,
            *work, 
            *rcos, 
            ddx, 
            ddy;

  int       nn, mm, kloc, nloc;
  
  ddx     = 1.0;
  ddy     = grid->delta_y/grid->delta_x;
  mm     = grid->n_cell_x  ; 
  nn     = grid->n_cell_y ; 

  rho11=dvector(0,mm*nn-1);
  rho22=dvector(0,mm*nn-1);
  work=dvector(0,mm+nn+2);
  rcos=dvector(0,mm*nn-1);

  for (kloc=0; kloc<mm; kloc++)
     for (nloc=0; nloc<nn; nloc++)
       {
	 if (kloc!=0 && nloc!=0) 
	   rcos[kloc*mm + nloc]=1.0/(2.0*((cos((double)(kloc)*PI/mm)-1.0)/(ddx*ddx)+(cos((double)(nloc)*PI/nn)-1.0)/(ddy*ddy)))*4.0/(mm*nn);
	 else
	   rcos[0]=0.;

        rho11[kloc*mm + nloc] = (4*PI)*(*(grid->rho1 + nloc*nn + kloc));
        rho22[kloc*mm + nloc] = (4*PI)*(*(grid->rho2 + nloc*nn + kloc));
      }

  potfft_(rho22,rcos,work,&mm,&nn); 
  potfft_(rho11,rcos,work,&mm,&nn);


  for (kloc=0; kloc<mm; kloc++)
    for (nloc=0; nloc<nn; nloc++)    
      { *(grid->phi1 + nloc*nn + kloc) = 1/ddy*rho11[kloc*mm + nloc];
        *(grid->phi2 + nloc*nn + kloc) = 1/ddy*rho22[kloc*mm + nloc];
     }

  free_dvector(rho11,0,mm*nn-1);
  free_dvector(rho22,0,mm*nn-1);
  free_dvector(work,0,mm*nn+2);
  free_dvector(rcos,0,mm*nn-1);

}

/******************************************************************************
 ****                            Main Program                              ****
 ******************************************************************************/

main ()

{ GRID *grid;

  printf ("                Program for studying head-tail instabilities due to\n");
  printf ("                               electron cloud                    \n\n");

  printf ("--> Reading data from configuration file.\n") ;
  read_data () ;

  printf ("--> Open files.\n") ;
  open_files ();
  printf ("\t\t Complete\n") ;

  printf ("--> Init values.\n") ;
  init_values () ;
  printf ("\t\t Complete\n") ;

  printf ("--> Generation of e- cloud and bunch distributions.\n");
  initialization ();
  printf ("\t\t Complete\n") ;

  if(i_method==2)
    { printf ("--> Generation of the grid for the particle-in-cell.\n");
      printf ("    calculation of the fields \n");

      grid=grid_init_comp(128,128,2);

      if(i_dip==2){
        gridextx = 3.*rpipex;
        gridexty = 3.*rpipey;
      }
      else{
        gridextx = 1.1*rpipex;
        gridexty = 1.1*rpipey;
      }
       if(i_bound==1){
          gridextx = 1.05*rpipex;
        gridexty = 1.05*rpipey;
       }




      grid_init_phys(grid,gridextx,gridexty);
      printf ("\t\t Complete\n") ;
    }

  fprintf (prot_inphase, "electrons = %d  protons = %d  bunch slices = %d\n",NEL,NPR,NBIN);
  fprintf (prot_inphase, "electron sizes -> sx = %g  sy = %g\n",sx2,sy2);
  fprintf (prot_inphase, "beam sizes     -> sx = %g  sy = %g\n",sx0,sy0);   
  fprintf (prot_inphase, "e- cloud sizes -> rpipex = %g  rpipey = %g\n",rpipex,rpipey);
  fprintf (prot_inphase, "maximum tune shifts -> dt_x = %g  dt_y = %g\n",dtunxmax,dtunymax);

  ili = 0;


  /* here the loop over the total number of steps (kicks * turns) 
     starts...                                                             */

  for(it=1; it<=nstep; it++)
    { if(npr_eff==NPR) 
         exit(2);

      xave = 0.0;
      yave = 0.0;
      zave = 0.0;
      xpave = 0.0;
      ypave = 0.0;
      dpmean = 0.0;
      sxsize = 0.0;
      sysize = 0.0;
      szsize = 0.0;
      xemit = 0.0;
      yemit = 0.0;
      corryz = 0.0;
      invary = 0.0;
      invarx = 0.0;
      for(imain=npr_eff; imain<NPR; imain++)
	{ xave += xpr[imain];
	  yave += ypr[imain];
          zave += sz[imain];
          dpmean += dp[imain];
          xpave += xpp[imain];
          ypave += ypp[imain];
	}
      xave /= (double)(NPR - npr_eff);
      yave /= (double)(NPR - npr_eff);
      zave /= (double)(NPR - npr_eff);
      xpave /= (double)(NPR - npr_eff);
      ypave /= (double)(NPR - npr_eff);
      dpmean /= (double)(NPR - npr_eff);
      for(imain=npr_eff; imain<NPR; imain++)
	{ sxsize += (xpr[imain] - xave)*(xpr[imain] - xave);
          sysize += (ypr[imain] - yave)*(ypr[imain] - yave);
          szsize += (sz[imain] - zave)*(sz[imain] - zave);
          sxpsize += (xpp[imain] - xpave)*(xpp[imain] - xpave);
          sypsize += (ypp[imain] - ypave)*(ypp[imain] - ypave); 
          xemit += (xpr[imain] - xave)*(xpp[imain] - xpave);
          yemit += (ypr[imain] - yave)*(ypp[imain] - ypave);
          corryz += (ypr[imain] - yave)*sz[imain];
          invary += (ypr[imain] - yave)*(ypr[imain] - yave) + betay0*betay0*(ypp[imain] - ypave)*(ypp[imain]-ypave);
          invarx += (xpr[imain] - xave)*(xpr[imain] - xave) + betax0*betax0*(xpp[imain] - xpave)*(xpp[imain]-xpave);

	}
      sxsize = sqrt(sxsize/(double)(NPR - npr_eff));
      sysize = sqrt(sysize/(double)(NPR - npr_eff));
      szsize = sqrt(szsize/(double)(NPR - npr_eff));
      sxpsize /= (double)(NPR - npr_eff);
      sypsize /= (double)(NPR - npr_eff);
      xemit /= (double)(NPR - npr_eff);
      yemit /= (double)(NPR - npr_eff);
      corryz /= (double)(NPR - npr_eff)*szsize;
      invarx /= (double)(NPR - npr_eff);
      invary /= (double)(NPR - npr_eff);
      timep = (double)(it - 1)*tstepp;

      //printf("sx0_cd = %lf (averaging on particles)", sqrt(sxsize));
     
      /* moments of the global transverse distribution have been calculated above 
         and after bining (to get bin_frac updated) will be printed to file             */

      /* every 20 turns we also store data for all four transverse 
         distributions of the bunch                                                     */ 
      
      /*      if (it % 20 == 0)
        { verteil (x,NPR,hdp,NDC,indxp);
      	  verteil (y,NPR,vdp,NDC,indyp);
          verteil (xp,NPR,hpdp,NDC,indxpp);
          verteil (yp,NPR,vpdp,NDC,indypp);
          for (lmain=1; lmain<=NDC; lmain++)
            fprintf(prot_distr,"%13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e\n",indxp[lmain],hdp[lmain],indyp[lmain],vdp[lmain],indxpp[lmain],hpdp[lmain],indypp[lmain],vpdp[lmain]);
	}
      */

      /* the electrons distribution is re-initialized before each interaction,
         but every 20 turns the deformed horizontal and vertical distributions
         are stored first                                                     
      
      if (it % 100 == 0)
        { verteil2 (xe,NEL,hde,NDC,indx);
      	  verteil2 (ye,NEL,vde,NDC,indy);
          for (lmain=1; lmain<=NDC; lmain++)
            fprintf(ele_distr,"%13.8e  %13.8e  %13.8e  %13.8e\n",indx[lmain],hde[lmain],indy[lmain],vde[lmain]);
	}
      */

      refresh_el ();

      //verteil (xe,NEL,hde,NDC,indx);
      //verteil (ye,NEL,vde,NDC,indy);
      //for (lmain=1; lmain<=NDC; lmain++)
      //  fprintf(prot_indistr,"%13.8e  %13.8e  %13.8e  %13.8e\n",indx[lmain],hde[lmain],indy[lmain],vde[lmain]);

      /* the beam must be sliced: each slice will then be made to interact
         with the electrons concentrated at one section                       */

      bining (zave,szsize);
 
      printf("%f\n",timep);
      fprintf(bunch_pr,"%13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e %13.8e %13.8e\n",timep,xave,xpave,yave,zave,dpmean,sxsize,sysize,szsize,sqrt(sxsize*sxsize*sxpsize-xemit*xemit),sqrt(sysize*sysize*sypsize-yemit*yemit),invarx/2/betax0,invary/2/betay0,corryz,(double)(NPR-npr_eff)/(double)NPR,bin_frac,sxs_ave,sys_ave);

      //printf("%13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e\n",timep,xave,yave,sqrt(sxsize),sqrt(sysize),sqrt(sxsize*sxpsize-xemit*xemit),sqrt(sysize*sypsize-yemit*yemit),invarx/2/betax0,invary/2/betay0,corryz);

     /* dispersion is added before interaction bunch-ecloud */ 

      if(dispx>0.1)
         for(jmain=npr_eff; jmain<NPR; jmain++)
	     xpr[jmain] += dispx * dp[jmain];
  
   
      /* here the initial position of the electrons is stored to be later
         used (in order to be able to extrapolate the effect of the cloud 
         distortion alone on the passing bunch particles and not the force
         due to the random initial distribution of the electrons)             

      for(imain=0; imain<NEL; imain++)
	{ xe0[imain] = xe[imain];
          ye0[imain] = ye[imain];
	}

         here the loop over the bunch slices starts... it is a decreasing
         loop since we want the particles with positive sz to be the head
         of the bunch                                                         */ 

      for(jmain=NBIN-1; jmain>=0; jmain--)
	{ sx = sxs[jmain];
	  sy = sys[jmain];
	  xoff = xs[jmain];
          yoff = ys[jmain];
          npr1 = npr[jmain];
    
          /* offsets and rms sizes of the slices are printed to file          */

	   if(it % n_diag == 0 || it == 1) 
            fprintf(headtail_pr, "%13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e\n",zs[jmain], npr1*xoff, npr1*yoff, npr1*sx, npr1*sy, npr1);

          if(i_dip==2 && i_ecool==0)
	    new_fraction ();

          /* here the loop over the beam-kicked electrons starts...           
             if the soft-Gaussian model is applied                            */ 

          if(i_ecool==0) {

	  switch (i_method)
	    { case 1:
              for(imain=0; imain<NEL; ++imain)
	         { xe[imain] += xpe[imain]*tstep;
  	           ye[imain] += ype[imain]*tstep;
                   xta = fabs(xe[imain] - xoff);
                   yta = fabs(ye[imain] - yoff);
                   x1 = xe[imain];
                   ips1 = ye[imain];
                   ffrank_(&x1,&ips1,&xoff,&yoff,&sx,&sy,&pre,&pim,&ili);
                   kick_xe = - pim * npr1 * factor1 * (xe[imain] - xoff)/xta * tstep;
                   kick_ye = - pre * npr1 * factor1 * (ye[imain] - yoff)/yta * tstep;

                   xpe[imain] += kick_xe;
                   ype[imain] += kick_ye;

              /* if the slice is not empty, each kicked electron acts back on
                 each particle in the slice (its having been kicked not being 
                 important at this stage, because the force with which it
                 acts, depends in fact only on its position                   */

              /* loop over the slice particles starts...                      */

                   if(iind[jmain]>0)
		     { for(kmain=0; kmain<iind[jmain]; kmain++)
		          { ind = indexp[jmain][kmain];
		            xof2 = xe[imain];
                            yof2 = ye[imain];
                            xta = fabs(xpr[ind] - xof2);
		            yta = fabs(ypr[ind] - yof2);
                            x2 = xpr[ind];
                            y2 = ypr[ind];
                            ffrank_(&x2,&y2,&xof2,&yof2,&sx2,&sy2,&pre,&pim,&ili);
                      //xof3 = xe0[imain];
                      //yof3 = ye0[imain];
                      //xta2 = fabs(x[ind] - xof3);
                      //yta2 = fabs(y[ind] - yof3);
                      //ffrank_(&x2,&y2,&xof3,&yof3,&sx2,&sy2,&pre2,&pim2,&ili);

		      /* the initial force is subtracted, thus the net force 
                         on particles in the first bunch slice is invariably
                         zero, because (xof2,yof2)=(xof3,yof3)                */
                         

                      //kick_x = -( pim * (x[ind] - xof2)/xta - pim2 *(x[ind] - xof3)/xta2) * qe0 * factor2 * tstep;
                      //kick_y = -( pre * (y[ind] - yof2)/yta - pre2 *(y[ind] - yof3)/yta2) * qe0 * factor2 * tstep;
                      
                            kick_x = -pim * (xpr[ind]-xof2)/xta * qe0 * factor2 * tstep;
                            kick_y = -pre * (ypr[ind]-yof2)/yta * qe0 * factor2 * tstep;

                            xpp[ind] += kick_x;
                            ypp[ind] += kick_y;
			  }
		     }
		 }

	      break;

	      case 2:
	      for(kmain=0; kmain<iind[jmain]; kmain++)
                 { vprx[kmain] = xpr[indexp[jmain][kmain]];
                   vpry[kmain] = ypr[indexp[jmain][kmain]];
                   vprxp[kmain] = xpp[indexp[jmain][kmain]];
                   vpryp[kmain] = ypp[indexp[jmain][kmain]];
		 }

              particles_distribute(grid,vprx,vpry,iind[jmain],1,qp0);
              particles_distribute(grid,xe,ye,NEL,2,qe0);

              /* calculate the forces */

              if(i_bound==0)
                field_calculate(grid);
              else
                field_poissonbc(grid);
 
              /* move the electrons - kick the beam particles */

              particles_move(grid,epr,vprx,vpry,vprxp,vpryp,iind[jmain],1,tstep,fscale2);
              particles_move(grid,eel,xe,ye,xpe,ype,NEL,2,tstep,fscale1);

              for(kmain=0; kmain<iind[jmain]; kmain++)
                 { xpr[indexp[jmain][kmain]] = vprx[kmain] ;
                   ypr[indexp[jmain][kmain]] = vpry[kmain] ;
                   xpp[indexp[jmain][kmain]] = vprxp[kmain] ;
                   ypp[indexp[jmain][kmain]] = vpryp[kmain] ;
	           //if (kmain % 100 == 0)
		   //fprintf(em2,"%13.8e  %13.8e  %13.8e  %13.8e\n",vprx[kmain],vprxp[kmain],vpry[kmain],vpryp[kmain]);
		 } 
              //for(kmain=0; kmain<NEL; kmain++)
		//if (kmain % 100 == 0)
		//fprintf(emiliano,"%13.8e  %13.8e  %13.8e  %13.8e\n",xe[kmain],xpe[kmain],ye[kmain],ype[kmain]);
	      break;

       	      case 0:
		break;
	    }
	  /*  and  now the kick due to the broad-band wake-field is added
              to the x' and y' of each proton in order to study the combined
              effect e-cloud + wake-field                                     */ 

          if (iind[jmain]>0 && iwake_flag == 1)
	    { for (kmain=0; kmain<iind[jmain]; kmain++)
	        { ind = indexp[jmain][kmain];
                  kick_x = 0;
		  kick_y = 0;
		  kick_z = wakefac*alphaz*rshz*1.e6*npr1;
                  if(i_pipe!=4) {
		    if (i_pipe!=5) {
		      for (mmain=NBIN-1; mmain>jmain; mmain--)
			{ zi = (jmain-mmain)*zstep;
			switch(i_pipe) {
			
                        case 0:
   		        kick_x += wakefac*npr[mmain]*xs[mmain]*wake_func(zi);
                        kick_y += wakefac*npr[mmain]*ys[mmain]*wake_func(zi);
                        kick_z += wakefac*npr[mmain]*wake_funcz(zi);
		        break;

		        case 1:
   		        kick_x += wakefac*npr[mmain]*0.41/1.23*(xs[mmain]-xpr[ind])*wake_func(zi);
                        kick_y += wakefac*npr[mmain]*0.82/1.23*(ys[mmain]+0.5*ypr[ind])*wake_func(zi);
                        kick_z += wakefac*npr[mmain]*wake_funcz(zi);
			break;
		      
                        case 2:
   		        kick_x += 0.;
                        kick_y += wakefac*npr[mmain]*ys[mmain]*wake_func(zi);
                        kick_z += wakefac*npr[mmain]*wake_funcz(zi);
			break;

                        case 3:
   		        kick_x += wakefac*npr[mmain]*xs[mmain]*wake_reswall(zi);
                        kick_y += wakefac*npr[mmain]*ys[mmain]*wake_reswall(zi);
                        kick_z += wakefac*npr[mmain]*wake_funcz(zi);
		        break;

			case 6:
   		        kick_x += wakefac*npr[mmain]*wake_reswall(zi)*(0.41*xs[mmain]-0.41*xpr[ind]);
                        kick_y += wakefac*npr[mmain]*wake_reswall(zi)*(0.82*ys[mmain]+0.41*ypr[ind]);
                        kick_z = 0.;       //prima era +=0 !!!!!!!!!
		        break;
			}  
			}
			//if(it==1 && kmain==0 && mmain==NBIN-1)
		        //fprintf(prot_distr,"%13.8e  %13.8e\n",zi,wake_func(merit,zetat,zi));

		    }
		    else
		      //(in the case i_pipe is equal to 5)
		      { ycoff =0.;     // b_coll/5. ;

			kick_x = 0.;
			kick_y +=-wakefac*1/(4*PI*EPS0)*(npr[jmain]/zstep)*(theta_coll*4/b_coll)*((ys[jmain]+ycoff-ypr[ind])+PI*h_coll/2/b_coll*ypr[ind]);
			// kick_y=kick_y*1e4;
			// kick_y=1e-10;
			// kick_y=1e-9*rand()/(double)RAND_MAX;
			// kick_y=1e-9*2.*(rand()/(double)RAND_MAX - 0.5);

			kick_z =0.;
	      
		      }
		  }
		  else  
		    //(in the case i_pipe is equal to 4)
                    kick_z = -9.57e-10 + 8.e-11*2.*(rand()/(double)RAND_MAX - 0.5);

                  xpp[ind] += kick_x;
                  ypp[ind] += kick_y;
                  dp[ind] += kick_z;
		}
	    }

	  // printf("%f %f %f\n",theta_coll,b_coll,h_coll);
	  //  printf("%f %f %f\n",kick_x,kick_y,kick_z);
	  //  printf("%f %d %f %f\n",wakefac,npr,zstep,PI );

          if (i_space == 1)

	    /*            { for(kmain=0; kmain<iind[jmain]; kmain++)
                 { vprx[kmain] = xpr[indexp[jmain][kmain]];
                   vpry[kmain] = ypr[indexp[jmain][kmain]];
                   vprxp[kmain] = xpp[indexp[jmain][kmain]];
                   vpryp[kmain] = ypp[indexp[jmain][kmain]];
		 }
  	      if (i_method == 0)
                { grid=grid_init_comp(128,128,2);
                  grid_init_phys(grid,5.*sx,5.*sy);
                  particles_distribute(grid,vprx,vpry,iind[jmain],1,qp0,xoff,yoff);
                  particles_distribute(grid,xe,ye,NEL,2,qe0,xave,yave);
		
              //calculate the forces
    
                  field_calculate(grid);
    
              //move the particles

                  particles_move(grid,epr,vprx,vpry,vprxp,vpryp,iind[jmain],2,tstep,fscale2,xoff,yoff);
                  particles_move(grid,eel,xe,ye,xpe,ype,NEL,1,tstep,fscalesp_ch,xave,yave);
		}
              else
                { field_calculate(grid);
                  particles_move(grid,epr,vprx,vpry,vprxp,vpryp,iind[jmain],2,tstep,fscale2,xoff,yoff);
                  particles_move(grid,eel,xe,ye,xpe,ype,NEL,1,tstep,fscalesp_ch,xave,yave);
		}

              for(kmain=0; kmain<iind[jmain]; kmain++)
                 { xpr[indexp[jmain][kmain]] = vprx[kmain] ;
                   ypr[indexp[jmain][kmain]] = vpry[kmain] ;
                   xpp[indexp[jmain][kmain]] = vprxp[kmain] ;
                   ypp[indexp[jmain][kmain]] = vpryp[kmain] ;
	           //if (kmain % 100 == 0)
		   //fprintf(em2,"%13.8e  %13.8e  %13.8e  %13.8e\n",vprx[kmain],vprxp[kmain],vpry[kmain],vpryp[kmain]);
		 } 
	    } */
             
              { for(kmain=0; kmain<iind[jmain]; kmain++)
	           { vprx[kmain] = xpr[indexp[jmain][kmain]];
                     vpry[kmain] = ypr[indexp[jmain][kmain]];
                     vprxp[kmain] = xpp[indexp[jmain][kmain]];
                     vpryp[kmain] = ypp[indexp[jmain][kmain]];
                     xta = fabs(vprx[kmain] - xoff);
                     yta = fabs(vpry[kmain] - yoff);
                     x1 = vprx[kmain];
                     ips1 = vpry[kmain];
                     ffrank_(&x1,&ips1,&xoff,&yoff,&sx,&sy,&pre,&pim,&ili);
                     kick_xp_sc = - pim * npr1 * factor2 * (vprx[kmain] - xoff)/xta * dlstep/gammar/gammar/C;
                     kick_yp_sc = - pre * npr1 * factor2 * (vpry[kmain] - yoff)/yta * dlstep/gammar/gammar/C;

                     vprxp[kmain] -= kick_xp_sc;
                     vpryp[kmain] -= kick_yp_sc;

                     xpr[indexp[jmain][kmain]] = vprx[kmain] ;
                     ypr[indexp[jmain][kmain]] = vpry[kmain] ;
                     xpp[indexp[jmain][kmain]] = vprxp[kmain] ;
                     ypp[indexp[jmain][kmain]] = vpryp[kmain] ;

		   }
	      }
	  }
          else {
	    // if i_ecool is not equal 0 ???
            refresh_el();
            for(kmain=0; kmain<iind[jmain]; kmain++)
               { vprx[kmain] = xpr[indexp[jmain][kmain]];
                 vpry[kmain] = ypr[indexp[jmain][kmain]];
                 vprxp[kmain] = xpp[indexp[jmain][kmain]];
                 vpryp[kmain] = ypp[indexp[jmain][kmain]];
	       }

            particles_distribute(grid,vprx,vpry,iind[jmain],1,qp0);

	    for(hmain=0;hmain<n_cool;hmain++){

              particles_distribute(grid,xe,ye,NEL,2,qe0);

              //printf("%d\n",hmain);
              /* calculate the forces */

              if(i_bound==0)
                field_calculate(grid);
              else
                field_poissonbc(grid);
 
              /* move the electrons - kick the beam particles */

              particles_move(grid,epr,vprx,vpry,vprxp,vpryp,iind[jmain],1,tstep,fscale2);
              particles_move(grid,eel,xe,ye,xpe,ype,NEL,2,tstep,fscale1);
	    }
              for(kmain=0; kmain<iind[jmain]; kmain++)
                 { xpr[indexp[jmain][kmain]] = vprx[kmain] ;
                   ypr[indexp[jmain][kmain]] = vpry[kmain] ;
                   xpp[indexp[jmain][kmain]] = vprxp[kmain] ;
                   ypp[indexp[jmain][kmain]] = vpryp[kmain] ;
		 } 

	  }  

	}  //closes the loop on bunch slices
    
           /* - i_rot = 1 the rotation is done around the full bunch centroid and using
               the overall bunch sigmas 
             - i_rot = 0 the rotation is done around the local centroid and by using 
	       the local sigmas                                                        */

      if(ishift==1 && i_rot==1)
        { //cent_x = xave;
  	  //cent_y = yave;
          //cent_xp = xpave;
          //cent_yp = ypave; 
      	  //dtunx = dtunxmax/sxsize/(sxsize+sysize);
      	  //dtuny = dtunymax/sysize/(sxsize+sysize);
	  cent_x = 0.;
  	  cent_y = 0.;
          cent_xp = 0.;
          cent_yp = 0.; 
          dtunx = dtunxmax;
          dtuny = dtunymax;
	}

      /* dispersion is subtracted for final transport of the bunch through the ring */ 

      if(dispx>0.1)
         for(jmain=npr_eff; jmain<NPR; jmain++) 
	     xpr[jmain] -= dispx * dp[jmain];

  
      /* loop for sextupolar kick in a section with non-zero dispersion             */

      if(i_sext==1)
         { for(jmain=npr_eff; jmain<NPR; jmain++)
	   {    xpp[jmain] += k_2s*((xpr[jmain]+dispx2*dp[jmain])*(xpr[jmain]+dispx2*dp[jmain])-ypr[jmain]*ypr[jmain]);
                ypp[jmain] += -k_2s*(xpr[jmain]+dispx2*dp[jmain])*ypr[jmain];
	   }
	 }
 
      if (i_damp==1){
      //randomnumber=(rand()/(double)RAND_MAX - 0.5);
      //printf("rand= %19.15e \n",randomnumber);
      dampx= - gainx*xave + noisex*(rand()/(double)RAND_MAX - 0.5);// randomnumber;  
      dampy= - gainy*yave + noisey*(rand()/(double)RAND_MAX - 0.5);

      }
      //

    if  (i_phas_adv!=0){

      gammaadv0=gammaadv;
      
      if (i_phas_adv==1)
	gammaadv=rand() / ((double)RAND_MAX);
      else if  (i_phas_adv==2)
	 gammaadv=phadv[nkick-(it%nkick)];

      //printf("%8.6e %8.6e \n",gammaadv0, gammaadv);

      hadv=2*PI*htune*(1.-gammaadv0)/(double)nkick + 2*PI*htune*gammaadv/(double)nkick;
      vadv=2*PI*vtune*(1.-gammaadv0)/(double)nkick + 2*PI*vtune*gammaadv/(double)nkick;
      
      cxa = cos(hadv);
      sxa = sin(hadv);
      cya = cos(vadv);
      sya = sin(vadv);
    }
	

	 

      /* loop over all the bunch particles starts, for the transformation of
         their phase space coordinates all through the piece of ring
         between two subsequent kicks...                                      */

      for(kmain=npr_eff; kmain<NPR; kmain++)
        { x0 = xpr[kmain];
          xp0 = xpp[kmain];          
          ips0 = ypr[kmain];
          yp0 = ypp[kmain];
          szz0 = sz[kmain];
	  dpp0 = dp[kmain];

	  /* - ishift=1 gives the space charge tune spread over the bunch particles
             - ishift=2 gives a random tune spread over the bunch particles centered
                        around the nominal tune and the maximum amplitude of which 
                        is the space charge induced one
	     - whatever other value of ishift produces no tune spread                  */

          switch(ishift){

	  case 0:

             xx1 = x0;
             xxp1 = xp0;
             yy1 = ips0;
             yyp1 = yp0;
             break;

          case 1:
             
	    //lambda = exp(-szz0*szz0/2/sz0/sz0);

             if(i_rot==0)
	      { sl_ind = (long)floor((szz0 + 2.*szsize - zave)/(4.*szsize)*NBIN);
                if (sl_ind >= 0 && sl_ind < NBIN)             
		 { cent_x = xs[sl_ind];
		   cent_y = ys[sl_ind];
                   cent_xp = xps[sl_ind];
                   cent_yp = yps[sl_ind]; 
                   dtunx = dtunxmax*sx0_cd*sx0_cd/sxs[sl_ind]/(sxs[sl_ind]+sys[sl_ind])*npr[sl_ind]/npr0*sqrt(2*PI)*sz0/zstep;
      	           dtuny = dtunymax*sy0*sy0/sys[sl_ind]/(sxs[sl_ind]+sys[sl_ind])*npr[sl_ind]/npr0*sqrt(2*PI)*sz0/zstep;
                   //printf("dtunx = %lf", dtunx);
                   //printf("dtuny = %lf", dtuny);
		   //dtunx = dtunxmax*sx0_cd*sx0_cd/sxs[sl_ind]/(sxs[sl_ind]+sys[sl_ind])*lambda;
		   //dtuny = dtunymax*sy0*sy0/sys[sl_ind]/(sxs[sl_ind]+sys[sl_ind])*lambda;
		 }
                else
		 { if (sl_ind < 0)
		     { cent_x = xs[0];
		       cent_y = ys[0];
                       cent_xp = xps[0];
                       cent_yp = yps[0]; 
      	               dtunx = dtunxmax*sx0_cd*sx0_cd/sxs[0]/(sxs[0]+sys[0])*npr[0]/npr0*sqrt(2*PI)*sz0/zstep;
      	               dtuny = dtunymax*sy0*sy0/sys[0]/(sxs[0]+sys[0])*npr[0]/npr0*sqrt(2*PI)*sz0/zstep;
		       //dtunx = dtunxmax*sx0_cd*sx0_cd/sxs[0]/(sxs[0]+sys[0])*lambda;
		       //dtuny = dtunymax*sy0*sy0/sys[0]/(sxs[0]+sys[0])*lambda;
		     }
                   else
		     { cent_x = xs[NBIN-1];
		       cent_y = ys[NBIN-1];
                       cent_xp = xps[NBIN-1];
                       cent_yp = yps[NBIN-1]; 
	      	       dtunx = dtunxmax*sx0_cd*sx0_cd/sxs[NBIN-1]/(sxs[NBIN-1]+sys[NBIN-1])*npr[NBIN-1]/npr0*sqrt(2*PI)*sz0/zstep;
      	               dtuny = dtunymax*sy0*sy0/sys[NBIN-1]/(sxs[NBIN-1]+sys[NBIN-1])*npr[NBIN-1]/npr0*sqrt(2*PI)*sz0/zstep;
		       //dtunx = dtunxmax*sx0_cd*sx0_cd/sxs[NBIN-1]/(sxs[NBIN-1]+sys[NBIN-1])*lambda;
		       //dtuny = dtunymax*sy0*sy0/sys[NBIN-1]/(sxs[NBIN-1]+sys[NBIN-1])*lambda;
		     }
		 }

	        xx1 = cos(2*PI*dtunx/(double)nkick)*(x0 - cent_x) - betax0*sin(2*PI*dtunx/(double)nkick)*(xp0 - cent_xp) + cent_x;
                xxp1 = 1/betax0*sin(2*PI*dtunx/(double)nkick)*(x0 - cent_x) + cos(2*PI*dtunx/(double)nkick)*(xp0 - cent_xp) + cent_xp;
	        yy1 = cos(2*PI*dtuny/(double)nkick)*(ips0 - cent_y) - betay0*sin(2*PI*dtuny/(double)nkick)*(yp0 - cent_yp) + cent_y;
                yyp1 = 1/betay0*sin(2*PI*dtuny/(double)nkick)*(ips0 - cent_y) + cos(2*PI*dtuny/(double)nkick)*(yp0 - cent_yp) + cent_yp;
	      }

             if(i_rot==1)
	      { xx1 = cos(2*PI*dtunx/(double)nkick)*(x0 - cent_x) - betax0*sin(2*PI*dtunx/(double)nkick)*(xp0 - cent_xp) + cent_x;
                xxp1 = 1/betax0*sin(2*PI*dtunx/(double)nkick)*(x0 - cent_x) + cos(2*PI*dtunx/(double)nkick)*(xp0 - cent_xp) + cent_xp;
	        yy1 = cos(2*PI*dtuny/(double)nkick)*(ips0 - cent_y) - betay0*sin(2*PI*dtuny/(double)nkick)*(yp0 - cent_yp) + cent_y;
                yyp1 = 1/betay0*sin(2*PI*dtuny/(double)nkick)*(ips0 - cent_y) + cos(2*PI*dtuny/(double)nkick)*(yp0 - cent_yp) + cent_yp;
	      }

	     break;

          case 2:

             lambda = 2*(rand ()/(double)RAND_MAX - 0.5);
             xx1 = cos(2*PI*dtunxmax*lambda/(double)nkick)*x0 - betax0*sin(2*PI*dtunxmax*lambda/(double)nkick)*xp0;
             xxp1 = 1/betax0*sin(2*PI*dtunxmax*lambda/(double)nkick)*x0 + cos(2*PI*dtunxmax*lambda/(double)nkick)*xp0;
	     yy1 = cos(2*PI*dtunymax*lambda/(double)nkick)*ips0 - betay0*sin(2*PI*dtunymax*lambda/(double)nkick)*yp0;
             yyp1 = 1/betay0*sin(2*PI*dtunymax*lambda/(double)nkick)*ips0 + cos(2*PI*dtunymax*lambda/(double)nkick)*yp0;

	     break;

	  }

	  //these lines about octupoles don't yet take into account the option for a random phase advance!!! 

          if(i_oct==1)
	    { epsx = (x0*x0 + xp0*xp0*betax0*betax0)/betax0;
              epsy = (ips0*ips0 + yp0*yp0*betay0*betay0)/betay0;

              // Remember that epsx = 2*actx;  epsy = 2*acty   

              cxa = cos(2*PI*(htune + qpxx*epsx + qpxy*epsy)/(double)nkick);
              sxa = sin(2*PI*(htune + qpxx*epsx + qpxy*epsy)/(double)nkick);
              cya = cos(2*PI*(vtune + qpxy*epsx + qpyy*epsy)/(double)nkick);
              sya = sin(2*PI*(vtune + qpxy*epsx + qpyy*epsy)/(double)nkick);

	    }

          if(i_oct==2)
	    { epsx = x0*x0/sxsize/sxsize;
              epsy = ips0*ips0/sysize/sysize;

              // Remember that epsx = 2*actx;  epsy = 2*acty     

              cxa = cos(2*PI*(htune + qpxx*epsx)/(double)nkick);
              sxa = sin(2*PI*(htune + qpxx*epsx)/(double)nkick);
              cya = cos(2*PI*(vtune + qpyy*epsy)/(double)nkick);
              sya = sin(2*PI*(vtune + qpyy*epsy)/(double)nkick);

	    }

	 if(i_oct==3)
	    { epsx =(x0*x0 + xp0*xp0*betax0*betax0)/sxsize/sxsize;
              epsy = (ips0*ips0 + yp0*yp0*betay0*betay0)/sysize/sysize;

	      // Remember that epsx = 2*actx;  epsy = 2*acty    

              cxa = cos(2*PI*(htune + qpxx*epsx + qpxy*epsy)/(double)nkick);
              sxa = sin(2*PI*(htune + qpxx*epsx + qpxy*epsy)/(double)nkick);
              cya = cos(2*PI*(vtune + qpxy*epsx + qpyy*epsy)/(double)nkick);
              sya = sin(2*PI*(vtune + qpxy*epsx + qpyy*epsy)/(double)nkick);

	    }



	  switch(i_incoh){

	  case 0:

	      if(ishift!=1)
               { sl_ind = (long)floor((szz0 + 2.*szsize)/(4.*szsize)*NBIN);
                 if (sl_ind >= 0 && sl_ind < NBIN)             
	           { cent_x = xs[sl_ind];
		     cent_y = ys[sl_ind];
                     cent_xp = xps[sl_ind];
                     cent_yp = yps[sl_ind]; 
		   }
                 else
		   { if (sl_ind < 0)
		       { cent_x = xs[0];
		         cent_y = ys[0];
                         cent_xp = xps[0];
                         cent_yp = yps[0]; 
		       }
                     else
		       { cent_x = xs[NBIN-1];
		         cent_y = ys[NBIN-1];
                         cent_xp = xps[NBIN-1];
                         cent_yp = yps[NBIN-1]; 
		       }
		   }
	       }

	       xx1 += -cent_x;
               xxp1 += -cent_xp;
	       yy1 += -cent_y;
               yyp1 += -cent_yp;
	       break;

	  case 2:

		 xx1 += -xave;
		 xxp1 += -xpave;
		 yy1 += -yave;
		 yyp1 += -ypave;
		 break;


	  case 1:
	       
	    if (i_damp==1  && (nkick==1 || it%nkick==1))
	      {
	      xx1 += dampx;
	      yy1 += dampy;
	      }
	    break;

	  }



	  /* next line introduces the possibility to have linear coupling in the machine */ 

	  if(i_coupl==1)
	    { xxp1 -= key_xy*yy1;
	      yyp1 -= key_xy*xx1;
	    }
	  

/*
	  //here we use a random phase advance, that is not exactely 1turn
	  switch (i_phas_adv) {

	    case 11:

	    gammaadv0=gammaadv;
	    gammaadv=rand() / ((double)RAND_MAX);
	    hadv=2*PI*htune*(1.-gammaadv0)/(double)nkick + 2*PI*htune*gammaadv/(double)nkick;
	    vadv=2*PI*vtune*(1.-gammaadv0)/(double)nkick + 2*PI*vtune*gammaadv/(double)nkick;
	    cxa = cos(hadv);
	    sxa = sin(hadv);
	    cya = cos(vadv);
	    sya = sin(vadv);
	    break;
	  
	  case 1:
	    if (kmain==npr_eff) {
	    gammaadv0=gammaadv;
	    gammaadv=rand() / ((double)RAND_MAX);
	    }
	    hadv=2*PI*htune*(1.-gammaadv0)/(double)nkick + 2*PI*htune*gammaadv/(double)nkick;
	    vadv=2*PI*vtune*(1.-gammaadv0)/(double)nkick + 2*PI*vtune*gammaadv/(double)nkick;
	    cxa = cos(hadv);
	    sxa = sin(hadv);
	    cya = cos(vadv);
	    sya = sin(vadv);
	    break;

	  case 0:

	    cxa = cos(2*PI*htune/(double)nkick);
	    sxa = sin(2*PI*htune/(double)nkick);
	    cya = cos(2*PI*vtune/(double)nkick);
	    sya = sin(2*PI*vtune/(double)nkick);
	    break;

	  case 2:
	    
	    gammaadv0=gammaadv;
	    gammaadv=phadv[nkick-(it%nkick)];
	   
	    hadv=2*PI*htune*(1.-gammaadv0)/(double)nkick + 2*PI*htune*gammaadv/(double)nkick;
	    vadv=2*PI*vtune*(1.-gammaadv0)/(double)nkick + 2*PI*vtune*gammaadv/(double)nkick;
	    cxa = cos(hadv);
	    sxa = sin(hadv);
	    cya = cos(vadv);
	    sya = sin(vadv);
	      
	    break;
	  }


	   if(i_oct==1)
	    { epsx = (x0*x0 + xp0*xp0*betax0*betax0)/betax0;
              epsy = (ips0*ips0 + yp0*yp0*betay0*betay0)/betay0;

              // Remember that epsx = 2*actx;  epsy = 2*acty    
              cxa = cos(2*PI*(htune + qpxx*epsx + qpxy*epsy)/(double)nkick);
              sxa = sin(2*PI*(htune + qpxx*epsx + qpxy*epsy)/(double)nkick);
              cya = cos(2*PI*(vtune + qpxy*epsx + qpyy*epsy)/(double)nkick);
              sya = sin(2*PI*(vtune + qpxy*epsx + qpyy*epsy)/(double)nkick);

	    }
 
	   if(i_oct==2)
	    { epsx = x0*x0/sxsize/sxsize;
              epsy = ips0*ips0/sysize/sysize;

              // Remember that epsx = 2*actx;  epsy = 2*acty     

              cxa = cos(2*PI*(htune + qpxx*epsx)/(double)nkick);
              sxa = sin(2*PI*(htune + qpxx*epsx)/(double)nkick);
              cya = cos(2*PI*(vtune + qpyy*epsy)/(double)nkick);
              sya = sin(2*PI*(vtune + qpyy*epsy)/(double)nkick);
	    }


	 if(i_oct==3)
	    { epsx =(x0*x0 + xp0*xp0*betax0*betax0)/sxsize/sxsize;
              epsy = (ips0*ips0 + yp0*yp0*betay0*betay0)/sysize/sysize;

	      // Remember that epsx = 2*actx;  epsy = 2*acty    

              cxa = cos(2*PI*(htune + qpxx*epsx + qpxy*epsy)/(double)nkick);
              sxa = sin(2*PI*(htune + qpxx*epsx + qpxy*epsy)/(double)nkick);
              cya = cos(2*PI*(vtune + qpxy*epsx + qpyy*epsy)/(double)nkick);
              sya = sin(2*PI*(vtune + qpxy*epsx + qpyy*epsy)/(double)nkick);

	    }


// fine random phase advance


*/


          r11 = cxa*cos(2*PI*(csix*dpp0+q_secx*dpp0*dpp0/2)/(double)nkick) - sxa * sin(2*PI*(csix*dpp0+q_secx*dpp0*dpp0/2)/(double)nkick);
          r12 = betax0*(sxa*cos(2*PI*(csix*dpp0+q_secx*dpp0*dpp0/2)/(double)nkick) + cxa * sin(2*PI*(csix*dpp0+q_secx*dpp0*dpp0/2)/(double)nkick));
          r21 = - r12/betax0/betax0;
          r22 = r11;

          /* horizontal coordinates transformation - linear and with
             chromatic rotation                                               */

          xpr[kmain] = xxp1 * r12 + xx1 * r11;
          xpp[kmain] = xxp1 * r22 + xx1 * r21;

          r33 = cya*cos(2*PI*(csiy*dpp0+q_secy*dpp0*dpp0/2)/(double)nkick) - sya * sin(2*PI*(csiy*dpp0+q_secy*dpp0*dpp0/2)/(double)nkick);
          r34 = betay0*(sya*cos(2*PI*(csiy*dpp0+q_secy*dpp0*dpp0/2)/(double)nkick) + cya * sin(2*PI*(csiy*dpp0+q_secy*dpp0*dpp0/2)/(double)nkick));
          r43 = - r34/betay0/betay0;
          r44 = r33;

          /* vertical coordinates transformation - linear and with
             chromatic rotation                                               */

          ypr[kmain] = yyp1 * r34 + yy1 * r33;
          ypp[kmain] = yyp1 * r44 + yy1 * r43;

          /* if one chooses to simulate synchrotron motion too, the
             longitudinal coordinates are transformed according to a
             focusing law or debunch or drift in a barrier bucket             */ 


          switch(isyn){

	  case 0:

  	     break;

          case 1:

      	     sz[kmain] = csa * szz0 - eta * C/omegas * ssa * dpp0;
	     dp[kmain] = csa * dpp0 + omegas/eta/C * ssa * szz0;
             break;

          case 2:

             sz[kmain] = szz0 - eta*circ*dpp0;
             break;

          case 3:

             sz[kmain] = szz0 - eta*circ*dpp0;
             if (fabs(sz[kmain]) >= 2*sz0)
	       { sz[kmain] = 4*sz0*fabs(sz[kmain])/sz[kmain] - sz[kmain];
	         dp[kmain] = -dp[kmain];
	       }
             break;

          case 4:

             kp1 = coeff3*sin(coeff4*szz0)*tstepp;
             k1 = coeff5*dpp0*tstepp;
             kp2 = coeff3*sin(coeff4*(szz0 + k1/2))*tstepp;
             k2 = coeff5*(dpp0 + kp1/2)*tstepp;
             kp3 = coeff3*sin(coeff4*(szz0 + k2/2))*tstepp;
             k3 = coeff5*(dpp0 + kp2/2)*tstepp;
             kp4 = coeff3*sin(coeff4*(szz0 + k3))*tstepp;
             k4 = coeff5*(dpp0 + kp3)*tstepp;
             sz[kmain] = szz0 + k1/6 + k2/3 + k3/3 + k4/6;
             dp[kmain] = dpp0 + kp1/6 + kp2/3 + kp3/3 + kp4/6;

             break;

          case 5:

               kp1 = (coeff3*sin(coeff4*szz0) + coeffspch*(szz0 - zave)/(szsize*szsize*szsize)*exp(-(szz0 - zave)*(szz0 - zave)/(2*szsize*szsize)))*tstepp;
               k1 = coeff5*dpp0*tstepp; 
               kp2 = (coeff3*sin(coeff4*(szz0 + k1/2)) + coeffspch*(szz0 - zave)/(szsize*szsize*szsize)*exp(-(szz0 - zave)*(szz0 - zave)/(2*szsize*szsize)))*tstepp;
               k2 = coeff5*(dpp0 + kp1/2)*tstepp; 
               kp3 = (coeff3*sin(coeff4*(szz0 + k2/2)) + coeffspch*(szz0 - zave)/(szsize*szsize*szsize)*exp(-(szz0 - zave)*(szz0 - zave)/(2*szsize*szsize)))*tstepp;
               k3 = coeff5*(dpp0 + kp2/2)*tstepp; 
               kp4 = (coeff3*sin(coeff4*(szz0 + k3)) + coeffspch*(szz0 - zave)/(szsize*szsize*szsize)*exp(-(szz0 - zave)*(szz0 - zave)/(2*szsize*szsize)))*tstepp;
               k4 = coeff5*(dpp0 + kp3)*tstepp; 
               sz[kmain] = szz0 + k1/6 + k2/3 + k3/3 + k4/6;
               dp[kmain] = dpp0 + kp1/6 + kp2/3 + kp3/3 + kp4/6;

             break;

          case 6:
	    /* for TESLA DR */
   
	     if (it%nkick== 1) {

             kp1 = coeff3*sin(coeff4*szz0)*tstepp*(double)nkick;
             k1 = coeff5*dpp0*tstepp*(double)nkick;
             kp2 = coeff3*sin(coeff4*(szz0 + k1/2))*tstepp*(double)nkick;
             k2 = coeff5*(dpp0 + kp1/2)*tstepp*(double)nkick;
             kp3 = coeff3*sin(coeff4*(szz0 + k2/2))*tstepp*(double)nkick;
             k3 = coeff5*(dpp0 + kp2/2)*tstepp*(double)nkick;
             kp4 = coeff3*sin(coeff4*(szz0 + k3))*tstepp*(double)nkick;
             k4 = coeff5*(dpp0 + kp3)*tstepp*(double)nkick;
             sz[kmain] = szz0 + k1 ;
             dp[kmain] = dpp0 + kp1*sz[kmain]/szz0 ;
	     /*             sz[kmain] = szz0 + k1/6 + k2/3 + k3/3 + k4/6;
			    dp[kmain] = dpp0 + kp1/6 + kp2/3 + kp3/3 + kp4/6; */

             }

             break;

          case 7:

	     /* for CLIC (or NLC?) DR */

	     if (it%(nkick/2)== 0) {
             k1 = coeff5*dpp0*tstepp/2.*(double)nkick;
             k2 = coeff5*(dpp0 + kp1/2)*tstepp/2.*(double)nkick;
             k3 = coeff5*(dpp0 + kp2/2)*tstepp/2.*(double)nkick;
             k4 = coeff5*(dpp0 + kp3)*tstepp/2.*(double)nkick;
	     sz[kmain] = szz0 + k1;
             }   

	     if (it%nkick== 0) {
             kp1 = coeff3*sin(coeff4*szz0)*tstepp*(double)nkick;
             /* kp2 = coeff3*sin(coeff4*(szz0 + k1/2))*tstepp*(double)nkick;
             kp3 = coeff3*sin(coeff4*(szz0 + k2/2))*tstepp*(double)nkick;
             kp4 = coeff3*sin(coeff4*(szz0 + k3))*tstepp*(double)nkick; */
             dp[kmain] = dpp0 + kp1*sz[kmain]/szz0 ;
	     /*	    dp[kmain] = dpp0 + kp1/6 + kp2/3 + kp3/3 + kp4/6; */
             }

             break;

          case 8:

	     /* for LHC */

             k1 = coeff5*dpp0*tstepp;
             k2 = coeff5*(dpp0 + kp1/2)*tstepp;
             k3 = coeff5*(dpp0 + kp2/2)*tstepp;
             k4 = coeff5*(dpp0 + kp3)*tstepp;
	     sz[kmain] = szz0 + k1;
   
	     if (it%nkick == 1) {
             kp1 = coeff3*sin(coeff4*szz0)*tstepp*(double)nkick;
             /* kp2 = coeff3*sin(coeff4*(szz0 + k1/2))*tstepp*(double)nkick;
             kp3 = coeff3*sin(coeff4*(szz0 + k2/2))*tstepp*(double)nkick;
             kp4 = coeff3*sin(coeff4*(szz0 + k3))*tstepp*(double)nkick; */
             dp[kmain] = dpp0 + kp1*sz[kmain]/szz0 ;
	     /*	    dp[kmain] = dpp0 + kp1/6 + kp2/3 + kp3/3 + kp4/6; */
             }

             break;


	  }

	  if (kmain % 300==0 && (it % n_diag==0 || it == 1))
	    fprintf(prot_phase,"%d \t%13.8e\t%13.8e\t%13.8e\t%13.8e\t%13.8e\t%13.8e\n",kmain,xpr[kmain],xpp[kmain],ypr[kmain],ypp[kmain],sz[kmain],dp[kmain]); 
	  else
	    if (kmain==NPR-1)
	      fprintf(samp_prot,"%d \t%13.8e\t%13.8e\t%13.8e\t%13.8e\t%13.8e\t%13.8e\n",it,xpr[kmain],xpp[kmain],ypr[kmain],ypp[kmain],sz[kmain],dp[kmain]);
	  //if (it % (n_diag/2)==0 )
	  // fprintf(track_pr,"%d %13.8e %13.8e \n",it,sz[kmain],dp[kmain]);
	 

	  //if (mark[kmain]==1)
	  //fprintf(track_pr,"%d  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e %d\n",kmain,xpr[kmain],xpp[kmain],ypr[kmain],ypp[kmain],sz[kmain],dp[kmain], it); 

	  //if (it==12 || it==24 ||it==36 || it==48 || it==60 || it==72 || it==84 || it==96 || it==120 )
	    // fprintf(prot_ini,"%d  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e  %13.8e\n",kmain,xpr[kmain],xpp[kmain],ypr[kmain],ypp[kmain],sz[kmain],dp[kmain]); 

	  /*          if(fabs(xpr[kmain])>=rpipex || fabs(ypr[kmain])>=rpipey || -0.5*coeff5/coeff3*coeff4*dp[kmain]*dp[kmain]-cos(coeff4*sz[kmain])>1) */
 
          if(i_loss==1) {
             if(fabs(xpr[kmain])>=rpipex || fabs(ypr[kmain])>=rpipey)
	       { //printf("jmain=%d, xpr[jmain]=%lf, xpp[jmain]=%lf\n",kmain,xpr[kmain],xpp[kmain]);
                 xpr[kmain] = xpr[npr_eff];
	         ypr[kmain] = ypr[npr_eff];
	         sz[kmain] = sz[npr_eff];
                 xpp[kmain] = xpp[npr_eff];
                 ypp[kmain] = ypp[npr_eff];
                 dp[kmain] = dp[npr_eff];
                 ++npr_eff;
	       }
	  }
}
      //    if (it % (n_diag/2)==0)
      //	{
        //fprintf(track_pr,"\n");
        //fprintf(track_pr,"\n");
      //}

      if (it % n_diag == 0 || it==1)
	 { if(isyn!=3)
             verteil (sz,NPR-npr_eff,hdp,NDC,indxp);
           else
             verteil2 (sz,NPR-npr_eff,hdp,NDC,indxp);

           for (lmain=0; lmain<NDC; lmain++)
                fprintf(prot_distr,"%13.8e  %13.8e\n",indxp[lmain],hdp[lmain]);
           fprintf(prot_distr,"\n");
           fprintf(prot_distr,"\n");  
           fprintf(headtail_pr,"\n");
           fprintf(headtail_pr,"\n");
           fprintf(prot_phase,"\n");
           fprintf(prot_phase,"\n");
           //fprintf(prot_ini,"\n");
           //fprintf(prot_ini,"\n");


	 }

    }

  fprintf (prot_inphase, "Percent of beam loss -> %g \n",(double)(npr_eff)/(double)(NPR)*100);

  fclose(bunch_pr);
  fclose(headtail_pr);
  fclose(prot_phase);
  fclose(prot_inphase);
  fclose(prot_ini);
  fclose(prot_distr);
  fclose(samp_prot);
    fclose(track_pr);
}












